from django import forms
from django.conf import settings
from django.contrib.admin.widgets import FilteredSelectMultiple
from django.db import models
from django.utils.translation import ugettext_lazy as _
from mptt.models import TreeForeignKey

from .models import Document, TrainingSet, Discipline
from .widgets import ManyToManyOverlay


class DisciplineChoiceField(forms.ModelMultipleChoiceField):
    """
    Custom form field in order to include parent nodes in string representation.
    Inherits from `forms.ModelMultipleChocieField`.
    """

    def label_from_instance(self, obj):
        if obj.parent:
            ancestors = [
                node.title for node in obj.parent.get_ancestors(include_self=True)
            ]
            ancestors.append(obj.title)
            return " \u2794 ".join(ancestors)
        else:
            return obj.title


class TrainingSetForm(forms.ModelForm):
    """
    Defining custom form for the training set admin interface.
    Inherits from `forms.ModelForm`.
    """

    class Meta:
        """
        Defining Meta description of `TrainingSetForm`.
        """

        model = TrainingSet
        fields = ["released", "title", "description", "icon", "documents"]

    title = models.CharField(max_length=255)
    description = models.CharField(max_length=255, blank=True)
    discipline = DisciplineChoiceField(
        queryset=Discipline.objects.all(),
        widget=FilteredSelectMultiple(
            verbose_name=(_("disciplines")), is_stacked=False
        ),
        label=_("disciplines"),
    )
    documents = forms.ModelMultipleChoiceField(
        queryset=Document.objects.all(),
        widget=ManyToManyOverlay(verbose_name=(_("vocabulary")), is_stacked=False),
        label=_("vocabulary"),
        help_text=_(
            "Please select some vocabularies for your training set. To see a preview of the corresponding image and audio files, press the alt key while selecting."
        ),
    )

    def clean(self):
        """
        Make sure the training set is only released when it contains at least as many vocabularies as defined in
        :attr:`~lunes_cms.core.settings.TRAININGSET_MIN_DOCS`.

        :return: The cleaned data for this form
        :rtype: dict
        """
        cleaned_data = super().clean()
        documents = cleaned_data.get("documents")
        if cleaned_data.get("released") and (
            not documents
            or documents.filter(document_image__confirmed=True).count()
            < settings.TRAININGSET_MIN_DOCS
        ):
            self.add_error(
                "released",
                _(
                    "You can only release a training set that contains at least {} vocabulary words with confirmed images."
                ).format(settings.TRAININGSET_MIN_DOCS),
            )
        return cleaned_data



from django.db import models
from mptt.models import MPTTModel, TreeForeignKey
from django.contrib.auth.models import Group
from django.contrib.contenttypes.fields import GenericRelation
from django.core.exceptions import ValidationError
from django.db.models.deletion import CASCADE
from django.utils.translation import ugettext_lazy as _
from django.utils.html import format_html

from ..utils import get_image_tag
from .feedback import Feedback
from .static import convert_umlaute_images
from .document import Document
from .discipline import Discipline


class TrainingSet(MPTTModel):  # pylint: disable=R0903
    """
    Training sets are part of disciplines, have a title, a description
    an icon and relates to documents and disciplines.
    Inherits from `mptt.models.MPTTModel`.
    """

    released = models.BooleanField(default=False, verbose_name=_("released"))
    title = models.CharField(max_length=255, verbose_name=_("training set"))
    description = models.CharField(
        max_length=255, blank=True, verbose_name=_("description")
    )
    icon = models.ImageField(
        upload_to=convert_umlaute_images, blank=True, verbose_name=_("icon")
    )
    documents = models.ManyToManyField(
        Document, related_name="training_sets", verbose_name=_("document")
    )
    discipline = models.ManyToManyField(
        Discipline, related_name="training_sets", verbose_name=_("discipline")
    )
    created_by = models.ForeignKey(
        Group, on_delete=CASCADE, null=True, blank=True, verbose_name=_("created by")
    )
    creator_is_admin = models.BooleanField(default=True, verbose_name=_("admin"))
    parent = TreeForeignKey(
        "self",
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        related_name="children",
        verbose_name=_("parent"),
    )
    feedback = GenericRelation(Feedback)

    def image_tag(self):
        """
        Image thumbnail to display a preview of the icon

        :return: img HTML tag to display an image thumbnail
        :rtype: str
        """
        return get_image_tag(self.icon)

    image_tag.short_description = ""

    def __str__(self):
        """String representation of TrainingSet instance

        :return: title of training set instance
        :rtype: str
        """
        return self.title

    def save(self, *args, **kwargs):
        """Overwrite djangos save function to assure
        that no child elements are created.

        :raises ValidationError: Exception if child training set is created
        """
        if self.parent:
            msg = _(
                "It is not possible to create child elements for training sets (unlike disciplines)."
            )
            raise ValidationError(msg)
        super(TrainingSet, self).save(*args, **kwargs)

    # pylint: disable=R0903
    class Meta:
        """
        Define user readable name of TrainingSet
        """

        verbose_name = _("training set")
        verbose_name_plural = _("training sets")

    def style_description_field(self):
        return format_html(
            '<div style="overflow-wrap: break-word; max-width: 150px;" >{}</div>',
            self.description,
        )

    style_description_field.short_description = "description"



# Generated by Django 3.2.13 on 2022-04-15 17:27

from django.db import migrations, models
import django.db.models.deletion
import lunes_cms.cms.models.static
import lunes_cms.cms.validators
import mptt.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Discipline",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "released",
                    models.BooleanField(default=False, verbose_name="released"),
                ),
                ("title", models.CharField(max_length=255, verbose_name="discipline")),
                (
                    "description",
                    models.CharField(
                        blank=True, max_length=255, verbose_name="description"
                    ),
                ),
                (
                    "icon",
                    models.ImageField(
                        blank=True,
                        upload_to=lunes_cms.cms.models.static.convert_umlaute_images,
                        verbose_name="icon",
                    ),
                ),
                (
                    "creator_is_admin",
                    models.BooleanField(default=True, verbose_name="admin"),
                ),
                ("lft", models.PositiveIntegerField(editable=False)),
                ("rght", models.PositiveIntegerField(editable=False)),
                ("tree_id", models.PositiveIntegerField(db_index=True, editable=False)),
                ("level", models.PositiveIntegerField(editable=False)),
                (
                    "created_by",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="discipline",
                        to="auth.group",
                        verbose_name="created by",
                    ),
                ),
                (
                    "parent",
                    mptt.fields.TreeForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="children",
                        to="cms.discipline",
                        verbose_name="parent",
                    ),
                ),
            ],
            options={
                "verbose_name": "discipline",
                "verbose_name_plural": "disciplines",
            },
        ),
        migrations.CreateModel(
            name="Document",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "word_type",
                    models.CharField(
                        choices=[
                            ("Nomen", "Substantiv"),
                            ("Verb", "Verb"),
                            ("Adjektiv", "Adjektiv"),
                        ],
                        default="",
                        max_length=255,
                        verbose_name="word type",
                    ),
                ),
                ("word", models.CharField(max_length=255, verbose_name="word")),
                (
                    "article",
                    models.IntegerField(
                        choices=[
                            (0, "keiner"),
                            (1, "der"),
                            (2, "die"),
                            (3, "das"),
                            (4, "die (Plural)"),
                        ],
                        default="",
                        verbose_name="article",
                    ),
                ),
                (
                    "audio",
                    models.FileField(
                        blank=True,
                        null=True,
                        upload_to=lunes_cms.cms.models.static.convert_umlaute_audio,
                        validators=[
                            lunes_cms.cms.validators.validate_file_extension,
                            lunes_cms.cms.validators.validate_file_size,
                            lunes_cms.cms.validators.validate_multiple_extensions,
                        ],
                        verbose_name="audio",
                    ),
                ),
                (
                    "creation_date",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="creation date"
                    ),
                ),
                (
                    "created_by",
                    models.CharField(
                        blank=True, max_length=255, null=True, verbose_name="created by"
                    ),
                ),
                (
                    "creator_is_admin",
                    models.BooleanField(default=True, verbose_name="admin"),
                ),
            ],
            options={
                "verbose_name": "vocabulary",
                "verbose_name_plural": "vocabulary",
            },
        ),
        migrations.CreateModel(
            name="TrainingSet",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "released",
                    models.BooleanField(default=False, verbose_name="released"),
                ),
                (
                    "title",
                    models.CharField(max_length=255, verbose_name="training set"),
                ),
                (
                    "description",
                    models.CharField(
                        blank=True, max_length=255, verbose_name="description"
                    ),
                ),
                (
                    "icon",
                    models.ImageField(
                        blank=True,
                        upload_to=lunes_cms.cms.models.static.convert_umlaute_images,
                        verbose_name="icon",
                    ),
                ),
                (
                    "creator_is_admin",
                    models.BooleanField(default=True, verbose_name="admin"),
                ),
                ("lft", models.PositiveIntegerField(editable=False)),
                ("rght", models.PositiveIntegerField(editable=False)),
                ("tree_id", models.PositiveIntegerField(db_index=True, editable=False)),
                ("level", models.PositiveIntegerField(editable=False)),
                (
                    "created_by",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="auth.group",
                        verbose_name="created by",
                    ),
                ),
                (
                    "discipline",
                    models.ManyToManyField(
                        related_name="training_sets",
                        to="cms.Discipline",
                        verbose_name="discipline",
                    ),
                ),
                (
                    "documents",
                    models.ManyToManyField(
                        related_name="training_sets",
                        to="cms.Document",
                        verbose_name="document",
                    ),
                ),
                (
                    "parent",
                    mptt.fields.TreeForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="children",
                        to="cms.trainingset",
                        verbose_name="parent",
                    ),
                ),
            ],
            options={
                "verbose_name": "training set",
                "verbose_name_plural": "training sets",
            },
        ),
        migrations.CreateModel(
            name="GroupAPIKey",
            fields=[
                (
                    "id",
                    models.CharField(
                        editable=False,
                        max_length=150,
                        primary_key=True,
                        serialize=False,
                        unique=True,
                    ),
                ),
                ("prefix", models.CharField(editable=False, max_length=8, unique=True)),
                ("hashed_key", models.CharField(editable=False, max_length=150)),
                ("created", models.DateTimeField(auto_now_add=True, db_index=True)),
                (
                    "name",
                    models.CharField(
                        default=None,
                        help_text="A free-form name for the API key. Need not be unique. 50 characters max.",
                        max_length=50,
                    ),
                ),
                (
                    "revoked",
                    models.BooleanField(
                        blank=True,
                        default=False,
                        help_text="If the API key is revoked, clients cannot use it anymore. (This cannot be undone.)",
                    ),
                ),
                (
                    "expiry_date",
                    models.DateTimeField(
                        blank=True,
                        help_text="Once API key expires, clients cannot use it anymore.",
                        null=True,
                        verbose_name="Expires",
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="api_keys",
                        to="auth.group",
                    ),
                ),
            ],
            options={
                "verbose_name": "API Key",
                "verbose_name_plural": "API Keys",
            },
        ),
        migrations.CreateModel(
            name="DocumentImage",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                ("name", models.CharField(blank=True, max_length=255)),
                (
                    "image",
                    models.ImageField(
                        upload_to=lunes_cms.cms.models.static.convert_umlaute_images,
                        validators=[
                            lunes_cms.cms.validators.validate_multiple_extensions
                        ],
                    ),
                ),
                (
                    "confirmed",
                    models.BooleanField(default=True, verbose_name="confirmed"),
                ),
                (
                    "document",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="document_image",
                        to="cms.document",
                    ),
                ),
            ],
            options={
                "verbose_name": "image",
                "verbose_name_plural": "images",
            },
        ),
        migrations.CreateModel(
            name="AlternativeWord",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "alt_word",
                    models.CharField(max_length=255, verbose_name="alternative word"),
                ),
                (
                    "article",
                    models.IntegerField(
                        choices=[
                            (0, "keiner"),
                            (1, "der"),
                            (2, "die"),
                            (3, "das"),
                            (4, "die (Plural)"),
                        ],
                        default="",
                        verbose_name="article",
                    ),
                ),
                (
                    "document",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="alternatives",
                        to="cms.document",
                    ),
                ),
            ],
            options={
                "verbose_name": "alternative word",
                "verbose_name_plural": "alternative words",
            },
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class ApiConfig(AppConfig):
    """
    Application settings for the `api` app,
    which is the app providing the REST API.
    Inherits from `AppConfig`.
    """

    name = "lunes_cms.api"
    verbose_name = _("API")



from rest_framework import serializers


class DocumentImageListSerializer(serializers.ListSerializer):
    """
    List Serializer for the DocumentImage module. Inherits from
    `serializers.ListSerializer`.
    """

    def to_representation(self, data):
        """
        Overwrite django built-in function to only deliver
        approved images.

        :param data: model instance
        :type data: models.Model
        :return: serialized model data
        :rtype: dict
        """
        data = data.filter(confirmed=True)
        return super(DocumentImageListSerializer, self).to_representation(data)



from .alternative_word import AlternativeWord
from .discipline import Discipline
from .document import Document
from .document_image import DocumentImage
from .static import Static, convert_umlaute_images, convert_umlaute_audio
from .training_set import TrainingSet
from .group_api_key import GroupAPIKey
from .feedback import Feedback
from . import group, content_type

__all__ = [
    "AlternativeWord",
    "Discipline",
    "Document",
    "DocumentImage",
    "Static",
    "TrainingSet",
    "convert_umlaute_images",
    "convert_umlaute_audio",
    "group",
    "GroupAPIKey",
    "Feedback",
]



from rest_framework import viewsets
from rest_framework.authentication import SessionAuthentication, BasicAuthentication
from rest_framework.permissions import IsAuthenticated

from ....cms.models import Document
from ..serializers import DocumentSerializer


class DocumentByIdViewSet(viewsets.ModelViewSet):
    """
    Defines a view set for the Document module of a given id.
    Inherits from `viewsets.ModelViewSet` and defines queryset
    and serializers.
    """

    serializer_class = DocumentSerializer
    http_method_names = ["get"]
    authentication_classes = [SessionAuthentication, BasicAuthentication]
    permission_classes = [IsAuthenticated]

    def get_queryset(self):
        """
        Defining custom queryset

        :param self: A handle to the :class:`DocumentViewSet`
        :type self: class

        :return: (filtered) queryset
        :rtype: QuerySet
        """
        if getattr(self, "swagger_fake_view", False):
            return Document.objects.none()
        user = self.request.user
        queryset = Document.objects.filter(id=self.kwargs["document_id"])
        return queryset



from django.contrib.auth.models import Group
from django.core.exceptions import PermissionDenied

from rest_framework import viewsets

from ....cms.models import GroupAPIKey, TrainingSet
from ...permissions import VerifyGroupKey
from ...utils import get_key
from ..serializers import GroupSerializer


class GroupViewSet(viewsets.ModelViewSet):
    """
    List available information of a user group.
    A valid API-Key is required.
    There is no need to pass a group id or similar,
    the returned queryset is filtered by the delivered API-Key.
    """

    permission_classes = [VerifyGroupKey]
    serializer_class = GroupSerializer
    http_method_names = ["get"]

    def get_queryset(self):
        """
        Defining custom queryset

        :param self: A handle to the :class:`GroupViewSet`
        :type self: class

        :return: (filtered) queryset
        :rtype: QuerySet
        """
        if getattr(self, "swagger_fake_view", False):
            return TrainingSet.objects.none()
        key = get_key(self.request)
        if not key:
            raise PermissionDenied()
        try:
            api_key_object = GroupAPIKey.get_from_token(key)
        except GroupAPIKey.DoesNotExist:
            raise PermissionDenied()
        if not api_key_object:
            raise PermissionDenied()
        queryset = Group.objects.filter(id=api_key_object.group_id)
        return queryset



from django.db import models
from django.utils.translation import ugettext_lazy as _

from PIL import Image, ImageFilter

from ..utils import get_image_tag
from ..validators import validate_multiple_extensions
from .static import Static, convert_umlaute_images
from .document import Document


class DocumentImage(models.Model):
    """Contains images and its titles that can be linked to
    a document object.
    """

    image = models.ImageField(
        upload_to=convert_umlaute_images, validators=[validate_multiple_extensions]
    )
    document = models.ForeignKey(
        Document, on_delete=models.CASCADE, related_name="document_image"
    )
    confirmed = models.BooleanField(default=True, verbose_name="confirmed")

    def image_tag(self):
        """
        Image thumbnail to display a preview of a image in the editing section
        of the DocumentImage admin.

        :return: img HTML tag to display an image thumbnail
        :rtype: str
        """
        return get_image_tag(self.image)

    image_tag.short_description = ""

    def save_original_img(self):
        """
        Function to save rough image with '_original' extension

        :param self: A handle to the :class:`model.DocumentImage`
        :type self: class: `model.DocumentImage`

        :return: None
        :rtype: None
        """
        name_elements = self.image.path.split(".")
        for elem in name_elements:
            if elem != name_elements[-1]:
                new_path = elem + "_"
        new_path = new_path + "original." + name_elements[-1]
        img = Image.open(self.image.path)
        img.save(new_path)

    def crop_img(self):
        """
        Function that crops the image and pastes it into a blurred background
        image

        :param self: A handle to the :class:`DocumentImage`
        :type self: class: `DocumentImage`

        :return: None
        :rtype: None
        """
        img_blurr = Image.open(self.image.path)
        img_cropped = Image.open(self.image.path)

        img_blurr = img_blurr.resize((Static.img_size[0], Static.img_size[1]))
        img_blurr = img_blurr.filter(ImageFilter.BoxBlur(Static.blurr_radius))

        if (
            img_cropped.width > Static.img_size[0]
            or img_cropped.height > Static.img_size[1]
        ):
            img_cropped.thumbnail(Static.img_size)
        elif (img_cropped.width / img_cropped.height) > (
            Static.img_size[0] / Static.img_size[1]
        ):
            img_cropped = img_cropped.resize(
                (
                    Static.img_size[0],
                    round(
                        (Static.img_size[0] / img_cropped.width) * img_cropped.height
                    ),
                )
            )
        else:
            img_cropped = img_cropped.resize(
                (
                    round(Static.img_size[1] / img_cropped.height) * img_cropped.width,
                    Static.img_size[1],
                )
            )

        offset = (
            ((img_blurr.width - img_cropped.width) // 2),
            ((img_blurr.height - img_cropped.height) // 2),
        )
        img_blurr.paste(img_cropped, offset)
        img_blurr.save(self.image.path)

    def __str__(self):
        """String representation of DocumentImage instance

        :return: title of document image instance
        :rtype: str
        """
        return self.document.word

    def save(self, *args, **kwargs):
        """Overwrite djangos save function to scale images into a
        uniform size that is defined in the Static module.
        """
        super(DocumentImage, self).save(*args, **kwargs)
        self.save_original_img()
        self.crop_img()

    class Meta:
        """
        Define user readable name of TrainingSet
        """

        verbose_name = _("image")
        verbose_name_plural = _("images")



"""
Lunes CMS URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
https://docs.djangoproject.com/en/2.2/topics/http/urls/

Examples:

Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')

Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')

Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))

"""
from django.contrib import admin
from django.contrib.auth import views as auth_views
from django.urls import path, include
from django.conf.urls.static import static
from django.conf import settings
from django.conf.urls.i18n import i18n_patterns
from django.conf.urls import url
from django.templatetags.static import static as get_static_url
from django.urls import path, reverse_lazy
from django.utils.translation import ugettext_lazy as _
from django.views.generic.base import RedirectView
from django.views.i18n import JavaScriptCatalog


#: The url patterns of this module (see :doc:`django:topics/http/urls`)
urlpatterns = [
    path("", RedirectView.as_view(url=reverse_lazy("admin:login"))),
    path(
        "favicon.ico",
        RedirectView.as_view(url=get_static_url("images/logo.svg")),
    ),
    path("api/", include("lunes_cms.api.urls", namespace="api")),
    path("", include("lunes_cms.help.urls")),
    url(r"^i18n/", include("django.conf.urls.i18n")),
    path("qr_code/", include("qr_code.urls", namespace="qr_code")),
]

urlpatterns += i18n_patterns(path("", include("lunes_cms.cms.urls")))

if settings.DEBUG:
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



from __future__ import absolute_import, unicode_literals
from django.contrib import admin
from django.db.models import F
from django.utils.translation import ugettext_lazy as _
from .models import Discipline, TrainingSet


class DisciplineListFilter(admin.SimpleListFilter):
    """
    Generic Filter for models that have a direct relationship to disciplines.
    Inherits from `admin.SimpleListFilter`.
    """

    title = _("disciplines")

    # Parameter for the filter that will be used in the URL query.
    parameter_name = "disciplines"

    def lookups(self, request, model_admin):
        """
        Defining look up values that can be seen in the admin
        interface. Returns tuples: the first element is a coded
        value, whereas the second one is human-readable.

        :param request: current user request
        :type request: django.http.request
        :param model_admin: admin of current model
        :type model_admin: ModelAdmin
        :return: list of tuples containing id and title of each discipline
        :rtype: list
        """
        list_of_disciplines = []

        # Verify that only disciplines are displayed that actually can contain training sets
        queryset = Discipline.objects.filter(lft=F("rght") - 1)

        if request.user.is_superuser:
            queryset = queryset.filter(creator_is_admin=True)
        else:
            queryset = queryset.filter(created_by__in=request.user.groups.all())
        for discipline in queryset:
            list_of_disciplines.append(
                (
                    str(discipline.id),
                    " \u2794 ".join(
                        map(str, discipline.get_ancestors(include_self=True))
                    ),
                )
            )
        return sorted(list_of_disciplines, key=lambda tp: tp[1])

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.

        :param request: current user request
        :type request: django.http.request
        :param queryset: current queryset
        :type queryset: QuerySet
        :return: filtered queryset based on the value provided in the query string
        :rtype: QuerySet
        """
        if self.value():
            return queryset.filter(discipline__id=self.value()).distinct()
        return queryset


class DocumentDisciplineListFilter(DisciplineListFilter):
    """
    Filter for disciplines within document list display.
    Inherits from `admin.SimpleListFilter`.
    """

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.

        :param request: current user request
        :type request: django.http.request
        :param queryset: current queryset
        :type queryset: QuerySet
        :return: filtered queryset based on the value provided in the query string
        :rtype: QuerySet
        """
        if self.value():
            return queryset.filter(
                training_sets__discipline__id=self.value()
            ).distinct()
        return queryset


class DocumentTrainingSetListFilter(admin.SimpleListFilter):
    """
    Filter for training sets within document list display.
    Inherits from `admin.SimpleListFilter`.
    """

    title = _("training sets")

    # Parameter for the filter that will be used in the URL query.
    parameter_name = "training set"

    def lookups(self, request, model_admin):
        """
        Defining look up values that can be seen in the admin
        interface. Returns tuples: the first element is a coded
        value, whereas the second one is human-readable.

        :param request: current user request
        :type request: django.http.request
        :param model_admin: admin of current model
        :type model_admin: ModelAdmin
        :return: list of tuples containing id and title of each training set
        :rtype: list
        """
        list_of_trainingsets = []
        if request.user.is_superuser:
            queryset = TrainingSet.objects.all().filter(creator_is_admin=True)
        else:
            queryset = TrainingSet.objects.all().filter(
                created_by__in=request.user.groups.all()
            )
        for trainingset in queryset:
            list_of_trainingsets.append((str(trainingset.id), trainingset.title))
        return sorted(list_of_trainingsets, key=lambda tp: tp[1])

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.

        :param request: current user request
        :type request: django.http.request
        :param queryset: current queryset
        :type queryset: QuerySet
        :return: filtered queryset based on the value provided in the query string
        :rtype: QuerySet
        """
        if self.value():
            return queryset.filter(training_sets__id=self.value()).distinct()
        return queryset


class ApprovedImageListFilter(admin.SimpleListFilter):
    """
    Filter for approved images within document list display.
    Inherits from `admin.SimpleListFilter`.
    """

    title = _("approved images")

    # Parameter for the filter that will be used in the URL query.
    parameter_name = "approvedimages"

    default_value = None

    def lookups(self, request, model_admin):
        """
        Defining look up values that can be seen in the admin
        interface. Returns tuples: the first element is a coded
        value, whereas the second one is human-readable

        :param request: current user request
        :type request: django.http.request
        :param model_admin: admin of current model
        :type model_admin: ModelAdmin
        :return: list of tuples containing id and title of each discipline
        :rtype: list
        """
        return (
            (1, _("at least one approved image")),
            (2, _("at least one pending image")),
            (3, _("no images")),
        )

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.

        :param request: current user request
        :type request: django.http.request
        :param queryset: current queryset
        :type queryset: QuerySet
        :return: filtered queryset based on the value provided in the query string
        :rtype: QuerySet
        """

        if self.value():
            if int(self.value()) == 1:
                return queryset.filter(document_image__confirmed=True).distinct()
            if int(self.value()) == 2:
                return queryset.filter(document_image__confirmed=False).distinct()
            if int(self.value()) == 3:
                return queryset.filter(document_image__isnull=True).distinct()
        return queryset


class AssignedListFilter(admin.SimpleListFilter):
    """
    Filter for documents that are either assigned or unassigned to at least one training set.
    Inherits from `admin.SimpleListFilter`.
    """

    title = _("assigned & unassigned")

    # Parameter for the filter that will be used in the URL query.
    parameter_name = "assigned"

    default_value = None

    def lookups(self, request, model_admin):
        """
        Defining look up values that can be seen in the admin
        interface. Returns tuples: the first element is a coded
        value, whereas the second one is human-readable

        :param request: current user request
        :type request: django.http.request
        :param model_admin: admin of current model
        :type model_admin: ModelAdmin
        :return: list of tuples containing id and title of each discipline
        :rtype: list
        """
        return (
            (0, _("unassigned only")),
            (1, _("assigned only")),
        )

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.

        :param request: current user request
        :type request: django.http.request
        :param queryset: current queryset
        :type queryset: QuerySet
        :return: filtered queryset based on the value provided in the query string
        :rtype: QuerySet
        """

        if self.value():
            if int(self.value()) == 0:
                return queryset.filter(training_sets__isnull=True).distinct()
            if int(self.value()) == 1:
                return queryset.filter(training_sets__isnull=False).distinct()
        return queryset



from django.contrib.auth.models import User, Group
from django.db.models.signals import post_save
from django.dispatch import receiver

from ..utils import create_ressource_path


class Static:
    """
    Module for static and global variables
    """

    # possible articles
    article_choices = [
        (0, "keiner"),
        (1, "der"),
        (2, "die"),
        (3, "das"),
        (4, "die (Plural)"),
    ]

    # possible word types
    word_type_choices = [
        ("Nomen", "Substantiv"),
        ("Verb", "Verb"),
        ("Adjektiv", "Adjektiv"),
    ]

    # number of pixles used for box blurr
    blurr_radius = 30
    # maximum (width, height) of images
    img_size = (1024, 768)

    # super admin group name
    admin_group = "Lunes"

    # default group name
    default_group_name = None


def convert_umlaute_images(instance, filename):
    """Convert file name of images to handle all kind of
    characters (inluding "Umlaute" etc.).

    :param instance: instance where the current file is being attached
    :type instance: django.db.models
    :param filename: name of the file
    :type filename: str
    :return: file path of converted image
    :rtype: str
    """
    return create_ressource_path("images", filename)


def convert_umlaute_audio(instance, filename):
    """Convert file name of audios to handle all kind of
    characters (inluding "Umlaute" etc.).

    :param instance: instance where the current file is being attached
    :type instance: django.db.models
    :param filename: name of the file
    :type filename: str
    :return: file path of converted audio
    :rtype: str
    """
    return create_ressource_path("audio", filename)


@receiver(post_save, sender=User)
def create_user_profile(sender, instance, created, **kwargs):
    """Automatically adds a group when creating a new user
    if group name given in Static module

    :param sender: user that sends request
    :type sender: django.contrib.auth.models
    :param instance: user that eventually will be added to a new group
    :type instance: django.contrib.auth.models
    :param created: checks if User is creator
    :type created: bool
    :return: False if User is not creator and not part of Static.default_group_name
    :rtype: bool
    """
    if Static.default_group_name:
        default_group = Group.objects.filter(name=Static.default_group_name)
        if not created or not default_group:
            return False
        instance.groups.add(Group.objects.get(name=Static.default_group_name))



"""
Map paths to view functions.
Defines custom schema views and a router that
handles the url patterns described in the `README.md` file
"""

from django.contrib import admin
from django.contrib.auth import views as auth_views
from django.urls import path, include
from django.conf.urls.static import static
from django.conf import settings
from django.conf.urls.i18n import i18n_patterns
from django.conf.urls import url
from django.templatetags.static import static as get_static_url
from django.urls import path, reverse_lazy
from django.utils.translation import ugettext_lazy as _
from django.views.generic.base import RedirectView
from django.views.i18n import JavaScriptCatalog


#: The url patterns of this module (see :doc:`django:topics/http/urls`)
urlpatterns = [
    path(
        "admin/password_reset/",
        auth_views.PasswordResetView.as_view(),
        name="admin_password_reset",
    ),
    path(
        "admin/password_reset/done/",
        auth_views.PasswordResetDoneView.as_view(),
        name="password_reset_done",
    ),
    path(
        "reset/<uidb64>/<token>/",
        auth_views.PasswordResetConfirmView.as_view(),
        name="password_reset_confirm",
    ),
    path(
        "reset/done/",
        auth_views.PasswordResetCompleteView.as_view(),
        name="password_reset_complete",
    ),
    path("admin/", admin.site.urls),
    path("i18n.js", JavaScriptCatalog.as_view(), name="javascript-translations"),
]

# Set dashboard title
admin.site.index_title = _("Dashboard")



from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError
import os


def validate_file_extension(value):
    """
    Function to validate the audio file format

    :param value: audio file returned by a `models.FileField`
    :type value: audio file

    :return: None
    :rtype: None
    """
    ext = os.path.splitext(value.name)[1]  # [0] returns path+filename
    valid_extensions = [".mp3", ".aac", ".wav", ".m4a", ".wma", ".ogg"]
    if not ext.lower() in valid_extensions:
        raise ValidationError(
            _("File type not supported! Use: .mp3 .aac .wav .m4a .wma .ogg")
        )


def validate_file_size(value):
    """
    Function to validate the size of an audio file

    :param value: audio file returned by a `models.FileField`
    :type value: audio file

    :return: None
    :rtype: None
    """
    if value.size > (5 * 1024 * 1024):
        raise ValidationError(_("File too large! Max. 5 MB"))


def validate_multiple_extensions(value):
    """
    Function that checks if an audio file has multiple extensions

    :param value: audio file returned by a `models.FileField`
    :type value: audio file

    :return: None
    :rtype: None
    """
    split_name = value.name.split(".")
    if len(split_name) != 2:
        raise ValidationError(_("Only use one file extension!"))



"""
Django settings for lunes-cms project.

Generated by 'django-admin startproject' using Django 2.2.

For more information on this file, see
https://docs.djangoproject.com/en/2.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.2/ref/settings/
"""

import os
from distutils.util import strtobool

from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext_lazy as _

from .logging_formatter import ColorFormatter


###################
# CUSTOM SETTINGS #
###################

#: Build paths inside the project like this: ``os.path.join(BASE_DIR, ...)``
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

#: How many documents a training sets needs at least to get released
TRAININGSET_MIN_DOCS = int(os.environ.get("LUNES_CMS_TRAININGSET_MIN_DOCS", 4))


########################
# DJANGO CORE SETTINGS #
########################

#: A boolean that turns on/off debug mode (see :setting:`django:DEBUG`)
#:
#: .. warning::
#:     Never deploy a site into production with :setting:`DEBUG` turned on!
DEBUG = bool(strtobool(os.environ.get("LUNES_CMS_DEBUG", "False")))

#: Enabled applications (see :setting:`django:INSTALLED_APPS`)
INSTALLED_APPS = [
    # Installed custom apps
    "lunes_cms.api",
    "lunes_cms.cms",
    "lunes_cms.help",
    # Django jazzmin needs to be installed before Django admin
    "jazzmin",
    # Installed Django apps
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.messages",
    "django.contrib.sessions",
    "django.contrib.staticfiles",
    # Installed third-party-apps
    "drf_spectacular",
    "mptt",
    "pydub",
    "rest_framework",
    "qr_code",
]

#: Activated middlewares (see :setting:`django:MIDDLEWARE`)
MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

#: Default URL dispatcher (see :setting:`django:ROOT_URLCONF`)
ROOT_URLCONF = "lunes_cms.core.urls"

#: Config for HTML templates (see :setting:`django:TEMPLATES`)
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "lunes_cms.core.context_processors.feedback_processor",
            ],
        },
    },
]

#: WSGI (Web Server Gateway Interface) config (see :setting:`django:WSGI_APPLICATION`)
WSGI_APPLICATION = "lunes_cms.core.wsgi.application"

#: The URL or named URL pattern where requests are redirected for login (see :setting:`django:LOGIN_URL`)
LOGIN_URL = "/admin/login/"

#: The URL or named URL pattern where requests are redirected after login when the
#: LoginView doesn't get a next GET parameter. (see :setting:`django:LOGIN_REDIRECT_URL`).
LOGIN_REDIRECT_URL = "/admin/"


############
# DATABASE #
############

#: A dictionary containing the settings for all databases to be used with this Django installation
#: (see :setting:`django:DATABASES`)
DATABASE_CHOICES = {
    "postgres": {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": os.environ.get("LUNES_CMS_DB_NAME", "lunes"),
        "USER": os.environ.get("LUNES_CMS_DB_USER", "lunes"),
        "PASSWORD": os.environ.get("LUNES_CMS_DB_PASSWORD", "password"),
        "HOST": os.environ.get("LUNES_CMS_DB_HOST", "localhost"),
        "PORT": os.environ.get("LUNES_CMS_DB_PORT", "5432"),
    },
    "sqlite": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    },
}

try:
    #: A dictionary containing the settings for all databases to be used with this Django installation
    #: (see :setting:`django:DATABASES`)
    DATABASES = {
        "default": DATABASE_CHOICES[os.environ.get("LUNES_CMS_DB_BACKEND", "postgres")]
    }
except KeyError as e:
    raise ImproperlyConfigured(
        f"The database backend {os.environ.get('LUNES_CMS_DB_BACKEND')!r} is not supported, must be one of {DATABASE_CHOICES.keys()}."
    ) from e

#: Default primary key field type to use for models that don’t have a field with primary_key=True.
DEFAULT_AUTO_FIELD = "django.db.models.AutoField"


############
# SECURITY #
############

#: This is a security measure to prevent HTTP Host header attacks, which are possible even under many seemingly-safe
#: web server configurations (see :setting:`django:ALLOWED_HOSTS` and :ref:`django:host-headers-virtual-hosting`)
ALLOWED_HOSTS = [".localhost", "127.0.0.1", "[::1]"] + list(
    filter(
        None,
        (x.strip() for x in os.environ.get("LUNES_CMS_ALLOWED_HOSTS", "").splitlines()),
    )
)

#: A list of IP addresses, as strings, that allow the :func:`~django.template.context_processors.debug` context
#: processor to add some variables to the template context.
INTERNAL_IPS = ["localhost", "127.0.0.1"]

#: The secret key for this particular Django installation (see :setting:`django:SECRET_KEY`)
#:
#: .. warning::
#:     Provide a key via the environment variable ``LUNES_CMS_SECRET_KEY`` in production and keep it secret!
SECRET_KEY = os.environ.get("LUNES_CMS_SECRET_KEY", "dummy" if DEBUG else "")

#: The list of validators that are used to check the strength of user's passwords.
#: See Password validation for more details. By default, no validation is performed and all passwords are accepted.
AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


########################
# INTERNATIONALIZATION #
########################

#: A list of directories where Django looks for translation files
#: (see :setting:`django:LOCALE_PATHS` and :doc:`django:topics/i18n/index`)
LOCALE_PATHS = [
    os.path.join(BASE_DIR, "locale"),
]

#: A string representing the language slug for this installation
#: (see :setting:`django:LANGUAGE_CODE` and :doc:`django:topics/i18n/index`)
LANGUAGE_CODE = "en"

#: A list of all available languages (see :setting:`django:LANGUAGES` and :doc:`django:topics/i18n/index`)
LANGUAGES = [
    ("en", _("English")),
    ("de", _("German")),
]

#: A string representing the time zone for this installation
#: (see :setting:`django:TIME_ZONE` and :doc:`django:topics/i18n/index`)
TIME_ZONE = "UTC"

#: A boolean that specifies whether Django’s translation system should be enabled
#: (see :setting:`django:USE_I18N` and :doc:`django:topics/i18n/index`)
USE_I18N = True

#: A boolean that specifies if localized formatting of data will be enabled by default or not
#: (see :setting:`django:USE_L10N` and :doc:`django:topics/i18n/index`)
USE_L10N = True

#: A boolean that specifies if datetimes will be timezone-aware by default or not
#: (see :setting:`django:USE_TZ` and :doc:`django:topics/i18n/index`)
USE_TZ = True


################
# STATIC FILES #
################

#: This setting defines the additional locations the :mod:`django.contrib.staticfiles` app will traverse to collect
#: static files for deployment or to serve them during development (see :setting:`django:STATICFILES_DIRS` and
#: :doc:`Managing static files <django:howto/static-files/index>`).
STATICFILES_DIRS = [os.path.join(BASE_DIR, "static")]

#: URL to use in development when referring to static files located in :setting:`STATICFILES_DIRS`
#: (see :setting:`django:STATIC_URL` and :doc:`Managing static files <django:howto/static-files/index>`)
STATIC_URL = "/static/"

#: The absolute path to the output directory where :mod:`django.contrib.staticfiles` will put static files for
#: deployment (see :setting:`django:STATIC_ROOT` and :doc:`Managing static files <django:howto/static-files/index>`)
#: In debug mode, this is not required since :mod:`django.contrib.staticfiles` can directly serve these files.
STATIC_ROOT = os.environ.get("LUNES_CMS_STATIC_ROOT")

#: URL that handles the media served from :setting:`MEDIA_ROOT` (see :setting:`django:MEDIA_URL`)
MEDIA_URL = "/media/"

#: Absolute filesystem path to the directory that will hold user-uploaded files (see :setting:`django:MEDIA_ROOT`)
MEDIA_ROOT = os.environ.get("LUNES_CMS_MEDIA_ROOT", os.path.join(BASE_DIR, "media"))


##########
# EMAILS #
##########

if DEBUG:
    #: The backend to use for sending emails (see :setting:`django:EMAIL_BACKEND` and :doc:`django:topics/email`)
    EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"
else:
    EMAIL_BACKEND = "django.core.mail.backends.smtp.EmailBackend"

#: The email address that error messages come from, such as those sent to :setting:`django:ADMINS`.
#: (see :setting:`django:SERVER_EMAIL`)
SERVER_EMAIL = os.environ.get("LUNES_CMS_SERVER_EMAIL", "keineantwort@lunes.app")

#: Default email address to use for various automated correspondence from the site manager(s)
#: (see :setting:`django:DEFAULT_FROM_EMAIL`)
DEFAULT_FROM_EMAIL = SERVER_EMAIL

#: The host to use for sending email (see :setting:`django:EMAIL_HOST`)
EMAIL_HOST = os.environ.get("LUNES_CMS_EMAIL_HOST", "localhost")

#: Password to use for the SMTP server defined in :attr:`~lunes_cms.core.settings.EMAIL_HOST`
#: (see :setting:`django:EMAIL_HOST_PASSWORD`). If empty, Django won’t attempt authentication.
EMAIL_HOST_PASSWORD = os.environ.get("LUNES_CMS_EMAIL_HOST_PASSWORD")

#: Username to use for the SMTP server defined in :attr:`~lunes_cms.core.settings.EMAIL_HOST`
#: (see :setting:`django:EMAIL_HOST_USER`). If empty, Django won’t attempt authentication.
EMAIL_HOST_USER = os.environ.get("LUNES_CMS_EMAIL_HOST_USER", SERVER_EMAIL)

#: Port to use for the SMTP server defined in :attr:`~lunes_cms.core.settings.EMAIL_HOST`
#: (see :setting:`django:EMAIL_PORT`)
EMAIL_PORT = int(os.environ.get("LUNES_CMS_EMAIL_PORT", 587))

#: Whether to use a TLS (secure) connection when talking to the SMTP server.
#: This is used for explicit TLS connections, generally on port 587.
#: (see :setting:`django:EMAIL_USE_TLS`)
EMAIL_USE_TLS = bool(strtobool(os.environ.get("LUNES_CMS_EMAIL_USE_TLS", "True")))

#: Whether to use an implicit TLS (secure) connection when talking to the SMTP server.
#: In most email documentation this type of TLS connection is referred to as SSL. It is generally used on port 465.
#: (see :setting:`django:EMAIL_USE_SSL`)
EMAIL_USE_SSL = bool(strtobool(os.environ.get("LUNES_CMS_EMAIL_USE_SSL", "False")))


###########
# LOGGING #
###########

#: The log level for lunes-cms django apps
LOG_LEVEL = os.environ.get("LUNES_CMS_LOG_LEVEL", "DEBUG" if DEBUG else "INFO")

#: The log level for the syslog
SYS_LOG_LEVEL = "INFO"

#: The log level for dependencies
DEPS_LOG_LEVEL = os.environ.get("LUNES_CMS_DEPS_LOG_LEVEL", "INFO" if DEBUG else "WARN")

#: The file path of the logfile. Needs to be writable by the application.
LOGFILE = os.environ.get("LUNES_CMS_LOGFILE", os.path.join(BASE_DIR, "lunes-cms.log"))

#: Logging configuration dictionary (see :setting:`django:LOGGING`)
LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "console": {
            "format": "{asctime} \x1b[1m{levelname}\x1b[0m {name} - {message}",
            "datefmt": "%b %d %H:%M:%S",
            "style": "{",
        },
        "console-colored": {
            "()": ColorFormatter,
            "format": "{asctime} {levelname} {name} - {message}",
            "datefmt": "%b %d %H:%M:%S",
            "style": "{",
        },
        "logfile": {
            "format": "{asctime} {levelname:7} {name} - {message}",
            "datefmt": "%b %d %H:%M:%S",
            "style": "{",
        },
        "syslog": {
            "format": "LUNES CMS - {levelname}: {message}",
            "style": "{",
        },
        "email": {
            "format": "Date and time: {asctime}\nSeverity: {levelname}\nLogger: {name}\nMessage: {message}\nFile: {funcName}() in {pathname}:{lineno}",
            "datefmt": "%Y-%m-%d %H:%M:%S",
            "style": "{",
        },
    },
    "filters": {
        "require_debug_true": {
            "()": "django.utils.log.RequireDebugTrue",
        },
        "require_debug_false": {
            "()": "django.utils.log.RequireDebugFalse",
        },
    },
    "handlers": {
        "console": {
            "filters": ["require_debug_true"],
            "class": "logging.StreamHandler",
            "formatter": "console",
        },
        "console-colored": {
            "filters": ["require_debug_true"],
            "class": "logging.StreamHandler",
            "formatter": "console-colored",
        },
        "logfile": {
            "class": "logging.FileHandler",
            "filename": LOGFILE,
            "formatter": "logfile",
        },
        "authlog": {
            "filters": ["require_debug_false"],
            "class": "logging.handlers.SysLogHandler",
            "address": "/dev/log",
            "facility": "auth",
            "formatter": "syslog",
        },
        "syslog": {
            "filters": ["require_debug_false"],
            "class": "logging.handlers.SysLogHandler",
            "address": "/dev/log",
            "facility": "syslog",
        },
    },
    "loggers": {
        # Loggers of lunes-cms django apps
        "lunes_cms": {
            "handlers": ["console-colored", "logfile"],
            "level": LOG_LEVEL,
            "propagate": False,
        },
        # Syslog for authentication
        "auth": {
            "handlers": ["console", "logfile", "authlog", "syslog"],
            "level": SYS_LOG_LEVEL,
            "propagate": False,
        },
        # Loggers of dependencies
        "django": {
            "handlers": ["console", "logfile"],
            "level": DEPS_LOG_LEVEL,
            "propagate": False,
        },
        "": {
            "handlers": ["console", "logfile"],
            "level": DEPS_LOG_LEVEL,
            "propagate": True,
        },
    },
}


#########################
# DJANGO REST FRAMEWORK #
#########################

#: Configuration of Django REST Framework
REST_FRAMEWORK = {
    # "DEFAULT_SCHEMA_CLASS": "rest_framework.schemas.coreapi.AutoSchema",
    "DEFAULT_SCHEMA_CLASS": "drf_spectacular.openapi.AutoSchema",
    "DEFAULT_VERSIONING_CLASS": "rest_framework.versioning.NamespaceVersioning",
    "ALLOWED_VERSIONS": ("v1", "v2"),
    "DEFAULT_VERSION": "default",
    "DEFAULT_API_URL": "http://localhost:8080/api/",
}

from django.templatetags.static import static

SPECTACULAR_SETTINGS = {
    "TITLE": "Lunes API",
    "DESCRIPTION": "The API documentation for the Lunes CMS",
    "VERSION": None,
    "SCHEMA_PATH_PREFIX": "/api(/v[0-9])?",
    "CONTACT": {"email": "tech@integreat-app.de"},
    "LICENSE": {
        "name": "Apache 2.0",
        "url": "https://www.apache.org/licenses/LICENSE-2.0.html",
    },
    "SWAGGER_UI_FAVICON_HREF": "/favicon.ico",
    "SWAGGER_UI_SETTINGS": """{
        deepLinking: true,
        presets: [SwaggerUIBundle.presets.apis, SwaggerUIStandalonePreset],
        layout: "StandaloneLayout",
    }""",
}

##################
# DJANGO JAZZMIN #
##################

#: Basic settings for Django Jazzmin
JAZZMIN_SETTINGS = {
    "site_brand": _("Lunes Administration"),
    "site_title": _("Lunes Administration"),
    "welcome_sign": _("Welcome to the vocabulary management of Lunes!"),
    "site_header": _("Lunes Administration"),
    "site_logo": "images/logo.svg",
    "site_icon": "images/logo.svg",
    "login_logo": "images/logo-lunes.svg",
    "login_logo_dark": "images/logo-lunes-dark.svg",
    "site_logo_classes": "",
    "changeform_format": "collapsible",
    "language_chooser": True,
    "custom_css": "css/corporate_identity.css",
    "custom_js": "js/corporate_identity.js",
    "icons": {
        "auth.user": "fas fa-user-edit",
        "auth.Group": "fas fa-users",
        "cms.Discipline": "fas fa-book",
        "cms.TrainingSet": "fas fa-stream",
        "cms.Document": "fab fa-amilia",
        "cms.GroupAPIKey": "fas fa-key",
        "cms.Feedback": "fas fa-comment",
    },
}

#: UI tweaks for Django Jazzmin
JAZZMIN_UI_TWEAKS = {
    "navbar_small_text": False,
    "footer_small_text": False,
    "body_small_text": False,
    "brand_small_text": False,
    "brand_colour": "navbar-dark",
    "accent": "accent-navy",
    "dark_mode_theme": "darkly",
    "navbar": "navbar-primary navbar-dark",
    "no_navbar_border": True,
    "navbar_fixed": False,
    "layout_boxed": False,
    "footer_fixed": False,
    "sidebar_fixed": False,
    "sidebar": "sidebar-dark-primary",
    "sidebar_nav_small_text": False,
    "sidebar_disable_expand": False,
    "sidebar_nav_child_indent": False,
    "sidebar_nav_compact_style": False,
    "sidebar_nav_legacy_style": True,
    "sidebar_nav_flat_style": False,
    "button_classes": {
        "primary": "btn-primary",
        "secondary": "btn-secondary",
        "info": "btn-info",
        "warning": "btn-warning",
        "danger": "btn-danger",
        "success": "btn-success",
    },
}


############
# QR CODES #
############

SERVE_QR_CODE_IMAGE_PATH = ""



from rest_framework import viewsets

from ....cms.models import Document, TrainingSet
from ...utils import check_group_object_permissions
from ..serializers import DocumentSerializer


class DocumentViewSet(viewsets.ModelViewSet):
    """
    List of available documents.
    A document is an item to be learned and consists of an image,
    multiple correct answers, and other details.
    If training set ID is provided as a parameter,
    the list will return only documents belonging to the training set.
    A valid API-Key may be required.
    """

    serializer_class = DocumentSerializer
    http_method_names = ["get"]

    def get_queryset(self):
        """
        Defining custom queryset

        :param self: A handle to the :class:`DocumentViewSet`
        :type self: class

        :return: (filtered) queryset
        :rtype: QuerySet
        """
        if getattr(self, "swagger_fake_view", False):
            return Document.objects.none()
        training_set_infos = TrainingSet.objects.filter(
            id=self.kwargs["training_set_id"]
        ).values_list("created_by_id", "creator_is_admin")
        group_id = training_set_infos[0][0]
        is_admin = training_set_infos[0][1]
        if group_id and not is_admin:
            check_group_object_permissions(self.request, group_id)
        queryset = (
            Document.objects.filter(
                training_sets__id=self.kwargs["training_set_id"],
                document_image__confirmed=True,
            )
            .select_related()
            .distinct()
            .order_by("word")
        )
        return queryset



"""
Register models for Django's CRUD back end and
specify autocomplete_fields, search_fields and nested modules
"""
from __future__ import absolute_import, unicode_literals

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from .models import Discipline, TrainingSet, Document, GroupAPIKey, Feedback
from .admins import DisciplineAdmin
from .admins import TrainingSetAdmin
from .admins import DocumentAdmin
from .admins import GroupAPIKeyAdmin
from .admins import FeedbackAdmin


def get_app_list(self, request):
    """
    Function that returns a sorted list of all the installed apps that have been
    registered in this site.

    :param self: A handle to the :class:`admin.AdminSite`
    :type self: class: `admin.AdminSite`
    :param request: current user request
    :type request: django.http.request

    :return: list of app dictionaries (e.g. containing models)
    :rtype: list
    """
    ordering = {
        _("disciplines").capitalize(): 1,
        _("training sets").capitalize(): 2,
        _("vocabulary").capitalize(): 3,
        _("api keys").capitalize(): 4,
        _("feedback").capitalize(): 5,
    }
    app_dict = self._build_app_dict(request)

    # Sort the apps alphabetically.
    app_list = sorted(app_dict.values(), key=lambda x: x["name"].lower())

    # Sort the respective modules according the defined order
    for app in app_list:
        try:
            app["models"].sort(key=lambda x: ordering[x["name"]])
        except KeyError:
            pass
    return app_list


admin.AdminSite.get_app_list = get_app_list
admin.site.register(
    Discipline,
    DisciplineAdmin,
    list_display=(
        "tree_actions",
        "indented_title",
        "released",
        "creator_group",
    ),
    list_display_links=("indented_title",),
)
admin.site.register(
    TrainingSet,
    TrainingSetAdmin,
    list_display=(
        "tree_actions",
        "indented_title",
        "title",
        "released",
        "related_disciplines",
        "creator_group",
    ),
    list_display_links=("indented_title",),
)
admin.site.register(Document, DocumentAdmin)
admin.site.register(GroupAPIKey, GroupAPIKeyAdmin)
admin.site.register(Feedback, FeedbackAdmin)



"""
This is the app which contains all API routes and classes which map the cms models to API JSON responses.
This is not the API documentation itself, but the Django developer documentation.
The usage documentation can be found here: https://lunes.tuerantuer.org/api/docs/
"""



from django.db import models
from django.utils.translation import ugettext_lazy as _

from .static import Static
from .document import Document


class AlternativeWord(models.Model):
    """
    Contains alternative words that can be linked to a document
    """

    alt_word = models.CharField(max_length=255, verbose_name=_("alternative word"))
    article = models.IntegerField(
        choices=Static.article_choices,
        default="",
        verbose_name=_("article"),
    )
    document = models.ForeignKey(
        Document, on_delete=models.CASCADE, related_name="alternatives"
    )

    def __str__(self):
        """String representation of AlternativeWord instance

        :return: title of alternative word instance
        :rtype: str
        """
        return self.alt_word

    # pylint: disable=R0903
    class Meta:
        """
        Define user readable name of Document
        """

        verbose_name = _("alternative word")
        verbose_name_plural = _("alternative words")



# Generated by Django 3.2.13 on 2022-05-21 09:49

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("cms", "0002_modify_group_model"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="documentimage",
            name="name",
        ),
    ]



from django.db import models
from django.db.models.deletion import CASCADE
from django.contrib.auth.models import Group
from django.contrib.contenttypes.fields import GenericRelation
from django.utils.translation import ugettext_lazy as _

from mptt.models import MPTTModel, TreeForeignKey

from ..utils import get_child_count, get_image_tag
from .feedback import Feedback
from .static import convert_umlaute_images


class Discipline(MPTTModel):
    """
    Disciplines for training sets.
    They have a title, a description, a icon and contain training
    sets with the same topic. Inherits from `mptt.models.MPTTModel`.
    """

    released = models.BooleanField(default=False, verbose_name=_("released"))
    title = models.CharField(max_length=255, verbose_name=_("discipline"))
    description = models.CharField(
        max_length=255, blank=True, verbose_name=_("description")
    )
    icon = models.ImageField(
        upload_to=convert_umlaute_images, blank=True, verbose_name=_("icon")
    )
    created_by = models.ForeignKey(
        Group,
        on_delete=CASCADE,
        null=True,
        blank=True,
        verbose_name=_("created by"),
        related_name="discipline",
    )
    creator_is_admin = models.BooleanField(default=True, verbose_name=_("admin"))
    parent = TreeForeignKey(
        "self",
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        related_name="children",
        verbose_name=_("parent"),
    )
    feedback = GenericRelation(Feedback)

    def is_valid(self):
        """Checks if a discipline itself or one of its children has at least one training set.
        If so, it is considered valid.

        :return: True if discipline is valid
        :rtype: bool
        """
        return (
            get_child_count(self) > 0
            or self.training_sets.filter(released=True).count() > 0
        )

    def get_nested_training_sets(self):
        """Returns a list of distinct training set ids that are part of this
        disipline or one of its child elements.

        :return: training set ids
        :rtype: list(int)
        """
        training_sets = []
        for child in self.get_descendants(include_self=True):
            training_sets += child.training_sets.filter(released=True).values_list(
                "id", flat=True
            )
        return set(training_sets)

    def image_tag(self):
        """
        Image thumbnail to display a preview of the icon

        :return: img HTML tag to display an image thumbnail
        :rtype: str
        """
        return get_image_tag(self.icon)

    image_tag.short_description = ""

    def __str__(self):
        """String representation of Discipline instance

        :return: title of discipline instance
        :rtype: str
        """
        return self.title

    class Meta:
        """
        Define user readable name of Field
        """

        verbose_name = _("discipline")
        verbose_name_plural = _("disciplines")



from rest_framework import serializers

from ....cms.models import AlternativeWord


class AlternativeWordSerializer(serializers.ModelSerializer):
    """
    Serializer for the AlternativeWord module. Inherits from
    `serializers.ModelSerializer`.
    """

    class Meta:
        """
        Define model and the corresponding fields
        """

        model = AlternativeWord
        fields = ("alt_word", "article")



from django.db.models import Count, Q

from rest_framework import viewsets

from ....cms.models import Discipline, TrainingSet
from ...utils import check_group_object_permissions
from ..serializers import TrainingSetSerializer


class TrainingSetViewSet(viewsets.ModelViewSet):
    """
    List training sets.
    If discipline ID is provided as a parameter,
    the list will return only training sets belonging to the discipline.
    A valid API-Key may be required.
    """

    serializer_class = TrainingSetSerializer
    http_method_names = ["get"]

    def get_queryset(self):
        """
        Defining custom queryset

        :param self: A handle to the :class:`TrainingSetViewSet`
        :type self: class

        :return: (filtered) queryset
        :rtype: QuerySet
        """
        if getattr(self, "swagger_fake_view", False):
            return TrainingSet.objects.none()
        discipline_infos = Discipline.objects.filter(
            id=self.kwargs["discipline_id"]
        ).values_list("created_by_id", "creator_is_admin")
        group_id = discipline_infos[0][0]
        is_admin = discipline_infos[0][1]
        if group_id and not is_admin:
            check_group_object_permissions(self.request, group_id)
        queryset = TrainingSet.objects.filter(
            discipline__id=self.kwargs["discipline_id"], released=True
        ).annotate(
            total_documents=Count(
                "documents",
                filter=Q(documents__document_image__confirmed=True),
                distinct=True,
            )
        )
        return queryset



from django.contrib import admin, messages
from django.utils.translation import gettext_lazy as _

from ..models import Feedback


class FeedbackAdmin(admin.ModelAdmin):
    """
    Admin interface for feedback.
    Inheriting from `admin.ModelAdmin`.
    """

    model = Feedback
    list_display = readonly_fields = [
        "comment",
        "content_object_link",
        "content_type",
        "created_date",
        "read_by",
    ]
    search_fields = ["comment"]
    list_filter = ["content_type", "read_by"]
    sortable_by = ["content_type", "created_date", "read_by"]
    actions = ["mark_as_read", "mark_as_unread"]

    def has_add_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    @admin.action(description=_("Mark as read"))
    def mark_as_read(self, request, queryset):
        """
        Action to mark selected items as read by user

        :param request: The current request
        :type request: ~django.http.HttpRequest

        :param queryset: The queryset of selected feedback entries
        :type queryset: ~django.db.models.query.QuerySet
        """
        queryset.update(read_by=request.user)
        messages.success(
            request,
            _(
                "The selected feedback entries were successfully marked as read.",
            ),
        )

    @admin.action(description=_("Mark as unread"))
    def mark_as_unread(self, request, queryset):
        """
        Action to mark selected items as unread

        :param request: The current request
        :type request: ~django.http.HttpRequest

        :param queryset: The queryset of selected feedback entries
        :type queryset: ~django.db.models.query.QuerySet
        """
        queryset.update(read_by=None)
        messages.success(
            request,
            _(
                "The selected feedback entries were successfully marked as unread.",
            ),
        )

    class Media:
        css = {"all": ("css/feedback.css",)}



from rest_framework import serializers

from ....cms.models import Document
from .alternative_word_serializer import AlternativeWordSerializer
from .document_image_serializer import DocumentImageSerializer


class DocumentSerializer(serializers.ModelSerializer):
    """
    Serializer for the Document module. Inherits from
    `serializers.ModelSerializer`.
    """

    alternatives = AlternativeWordSerializer(many=True, read_only=True)
    document_image = DocumentImageSerializer(many=True, read_only=True)

    class Meta:
        """
        Define model and the corresponding fields
        """

        model = Document
        fields = (
            "id",
            "word",
            "article",
            "audio",
            "word_type",
            "alternatives",
            "document_image",
        )



from __future__ import absolute_import, unicode_literals

from django.conf import settings
from django.contrib import admin, messages
from django.db.models import Count, F, Q
from django.utils.translation import ugettext_lazy as _, ngettext

from mptt.admin import DraggableMPTTAdmin

from ..list_filter import DisciplineListFilter
from ..forms import TrainingSetForm
from ..models import Static, Document, Discipline
from ..utils import iter_to_string


class TrainingSetAdmin(DraggableMPTTAdmin):
    """
    Admin Interface to for the TrainigSet module.
    Inheriting from `mptt.admin.DraggableMPTTAdmin`.
    """

    exclude = ("creator_is_admin",)
    fields = [
        "released",
        "title",
        "description",
        "icon",
        "image_tag",
        "documents",
        "discipline",
        "created_by",
    ]
    readonly_fields = ["created_by", "image_tag"]
    search_fields = ["title"]
    form = TrainingSetForm
    list_filter = (DisciplineListFilter,)
    actions = ["make_released", "make_unreleased"]
    list_per_page = 25

    def save_model(self, request, obj, form, change):
        """
        Overwrite django built-in function to save
        user group and admin satus of model

        :param request: current user request
        :type request: django.http.request
        :param obj: training set object
        :type obj: models.TrainingSet
        :param form: employed model form
        :type form: ModelForm
        :param change: True if change on existing model
        :type change: bool
        :raises IndexError: Error when user is not superuser and doesn't belong to any group
        """
        if not change:
            if len(request.user.groups.all()) >= 1:
                obj.created_by = request.user.groups.all()[0]
            elif not request.user.is_superuser:
                raise IndexError("No group assigned. Please add the user to a group")
            obj.creator_is_admin = request.user.is_superuser
        obj.save()

    def get_action_choices(self, request):
        """
        Overwrite django built-in function to modify action choices. The first
        option is dropped since it is a place holder.

        :param request: current user request
        :type request: django.http.request
        :return: modified action choices
        :rtype: dict
        """
        choices = super(TrainingSetAdmin, self).get_action_choices(request)
        choices.pop(0)
        return choices

    def get_queryset(self, request):
        """
        Overwrite django built-in function to modify queryset according to user.
        Users that are not superusers only see training set of their groups.

        :param request: current user request
        :type request: django.http.request
        :return: adjustet queryset
        :rtype: QuerySet
        """
        qs = super(TrainingSetAdmin, self).get_queryset(request)
        if request.user.is_superuser:
            return qs.filter(creator_is_admin=True)
        return qs.filter(created_by__in=request.user.groups.all())

    def get_form(self, request, obj=None, **kwargs):
        """
        Overwrite django built-in function to define custom choices
        in many to many selectors, e.g. users should not see documents
        by superusers. The function modifies the querysets of the
        corresponding base fields dynamically.

        :param request: current user request
        :type request: django.http.request
        :param obj: django model object, defaults to None
        :type obj: django.db.models, optional
        :return: model form with adjustet querysets
        :rtype: ModelForm
        """
        form = super(TrainingSetAdmin, self).get_form(request, obj, **kwargs)
        if not request.user.is_superuser:
            form.base_fields["discipline"].queryset = (
                Discipline.objects.filter(
                    created_by__in=request.user.groups.all(), lft=F("rght") - 1
                )
                .order_by("title")
                .order_by("level")
            )
            form.base_fields["documents"].queryset = (
                Document.objects.filter(
                    Q(created_by__in=request.user.groups.all())
                    | (
                        Q(creator_is_admin=True, document_image__confirmed=True)
                        & ~Q(audio="")
                    )
                )
                .distinct()
                .order_by("word")
            )
        else:
            form.base_fields["discipline"].queryset = (
                Discipline.objects.filter(creator_is_admin=True, lft=F("rght") - 1)
                .order_by("title")
                .order_by("level")
            )
            form.base_fields["documents"].queryset = Document.objects.filter(
                creator_is_admin=True
            ).order_by("word")
        return form

    @admin.action(description=_("Release selected training sets"))
    def make_released(self, request, queryset):
        """
        Action to release training set objects. It sets the
        corresponding boolean field to true.

        :param request: current user request
        :type request: django.http.request
        :param queryset: current queryset
        :type queryset: QuerySet
        """
        # Annotate queryset so it can be filtered by the document count later
        annotated_queryset = queryset.annotate(
            document_count=Count(
                "documents",
                filter=Q(documents__document_image__confirmed=True),
                distinct=True,
            )
        )
        # Split training sets by their validity and their "released" status
        invalid_trainingsets = []
        released_trainingsets = []
        unreleased_trainingsets = []
        for trainingset in annotated_queryset:
            if trainingset.document_count < settings.TRAININGSET_MIN_DOCS:
                invalid_trainingsets.append(trainingset)
            elif trainingset.released:
                released_trainingsets.append(trainingset)
            else:
                unreleased_trainingsets.append(trainingset)
        # Show error message for invalid trainingsets
        if invalid_trainingsets:
            messages.error(
                request,
                ngettext(
                    "The training set {} could not be released because it contains less than {} vocabulary words with confirmed images.",
                    "The training sets {} could not be released because they contain less than {} vocabulary words with confirmed images.",
                    len(invalid_trainingsets),
                ).format(
                    iter_to_string(invalid_trainingsets),
                    settings.TRAININGSET_MIN_DOCS,
                ),
            )
        # Show info message for valid trainingsets that are already released
        if released_trainingsets:
            messages.info(
                request,
                ngettext(
                    "The training set {} is already released.",
                    "The training sets {} are already released.",
                    len(released_trainingsets),
                ).format(iter_to_string(released_trainingsets)),
            )
        # Update valid trainingsets and show success message
        if unreleased_trainingsets:
            # Update valid queryset
            annotated_queryset.filter(
                document_count__gte=settings.TRAININGSET_MIN_DOCS
            ).update(released=True)
            messages.success(
                request,
                ngettext(
                    "The training set {} was successfully released.",
                    "The training sets {} were successfully released.",
                    len(unreleased_trainingsets),
                ).format(iter_to_string(unreleased_trainingsets)),
            )

    @admin.action(description=_("Unrelease selected training sets"))
    def make_unreleased(self, request, queryset):
        """
        Action to hide discipline objects. It sets the
        corresponding boolean field to false.

        :param request: current user request
        :type request: django.http.request
        :param queryset: current queryset
        :type queryset: QuerySet
        """
        # Split all training sets by their "released" status
        released_trainingsets = []
        unreleased_trainingsets = []
        for trainingset in queryset:
            if trainingset.released:
                released_trainingsets.append(trainingset)
            else:
                unreleased_trainingsets.append(trainingset)
        # Add info messages for training sets that are already unreleased
        if unreleased_trainingsets:
            messages.info(
                request,
                ngettext(
                    "The training set {} is already unreleased.",
                    "The training sets {} are already unreleased.",
                    len(unreleased_trainingsets),
                ).format(iter_to_string(unreleased_trainingsets)),
            )
        # Get all training sets that are released and need to be updated
        if released_trainingsets:
            queryset.update(released=False)
            messages.success(
                request,
                ngettext(
                    "The training set {} was successfully unreleased.",
                    "The training sets {} were successfully unreleased.",
                    len(released_trainingsets),
                ).format(iter_to_string(released_trainingsets)),
            )

    def related_disciplines(self, obj):
        """
        Display related disciplines in list display

        :param obj: Training set object
        :type obj: models.TrainingSet
        :return: comma seperated list of related disciplines
        :rtype: str
        """
        return ", ".join([child.title for child in obj.discipline.all()])

    related_disciplines.short_description = _("disciplines")

    def creator_group(self, obj):
        """
        Include creator group of discipline in list display

        :param obj: Training set object
        :type obj: models.TrainingSet
        :return: Either static admin group or user group
        :rtype: str
        """
        if obj.creator_is_admin:
            return Static.admin_group
        elif obj.created_by:
            return obj.created_by
        else:
            return None

    creator_group.short_description = _("creator group")

    class Media:
        js = ("js/image_preview.js",)



from django.core.exceptions import PermissionDenied
from django.db.models import Q
from django.utils.timezone import now

from rest_framework import permissions

from ..cms.models import GroupAPIKey
from .utils import get_key


class VerifyGroupKey(permissions.AllowAny):
    """Simple permissions class that blocks
    requests if no valid API-Token is delivered.
    Inherits from `permissions.AllowAny`.
    """

    def has_permission(self, request, view):
        """Checks whether a valid API-Key is send
        by the user in the authorization header

        :param request: http request
        :type request: HttpRequest
        :param view: restframework view
        :type view: viewsets.ModelViewSet
        :return: False if user doesn't send a API-key
        :rtype: bool
        """
        key = get_key(request)
        if key is None:
            return False
        GroupAPIKey.get_from_token(key)
        return True



from rest_framework import serializers

from ....cms.models import TrainingSet
from .fallback_icon_serializer import FallbackIconSerializer


class TrainingSetSerializer(FallbackIconSerializer):
    """
    Serializer for the TrainingSet module. Inherits from
    `FallbackIconSerializer`.
    """

    total_documents = serializers.IntegerField()

    class Meta:
        """
        Define model and the corresponding fields
        """

        model = TrainingSet
        fields = ("id", "title", "description", "icon", "total_documents")



# Generated by Django 3.2.13 on 2022-06-13 08:20

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("cms", "0003_remove_documentimage_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="Feedback",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "object_id",
                    models.PositiveIntegerField(
                        help_text="The id of the object this feedback entry refers to.",
                        verbose_name="object id",
                    ),
                ),
                ("comment", models.TextField(verbose_name="comment")),
                (
                    "created_date",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="The time and date when the feedback was submitted.",
                        verbose_name="submitted on",
                    ),
                ),
                (
                    "content_type",
                    models.ForeignKey(
                        help_text="The content type this feedback entry refers to.",
                        limit_choices_to=models.Q(
                            ("app_label", "cms"),
                            ("model__in", ["discipline", "trainingset", "document"]),
                        ),
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.contenttype",
                        verbose_name="content type",
                    ),
                ),
                (
                    "read_by",
                    models.ForeignKey(
                        blank=True,
                        help_text="The user who marked this feedback as read. If the feedback is unread, this field is empty.",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="feedback",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="marked as read by",
                    ),
                ),
            ],
            options={
                "verbose_name": "feedback",
                "verbose_name_plural": "feedback entries",
            },
        ),
    ]



import os

from pathlib import Path
from django.core.files import File
from django.contrib.contenttypes.fields import GenericRelation
from django.db import models
from django.utils.translation import ugettext_lazy as _

from pydub import AudioSegment

from ..utils import document_to_string
from ..validators import (
    validate_file_extension,
    validate_file_size,
    validate_multiple_extensions,
)
from .feedback import Feedback
from .group import Group
from .static import Static, convert_umlaute_audio


class Document(models.Model):
    """
    Contains a word type, a word, an article and an audio.
    Relates to training sets and inherits from `models.Model`.
    """

    word_type = models.CharField(
        max_length=255,
        choices=Static.word_type_choices,
        default="",
        verbose_name=_("word type"),
    )
    word = models.CharField(max_length=255, verbose_name=_("word"))
    article = models.IntegerField(
        choices=Static.article_choices,
        default="",
        verbose_name=_("article"),
    )
    audio = models.FileField(
        upload_to=convert_umlaute_audio,
        validators=[
            validate_file_extension,
            validate_file_size,
            validate_multiple_extensions,
        ],
        blank=True,
        null=True,
        verbose_name=_("audio"),
    )
    creation_date = models.DateTimeField(
        auto_now_add=True, verbose_name=_("creation date")
    )
    created_by = models.ForeignKey(
        max_length=255,
        null=True,
        blank=True,
        verbose_name=_("created by"),
        to=Group,
        on_delete=models.CASCADE,
    )
    creator_is_admin = models.BooleanField(default=True, verbose_name=_("admin"))
    feedback = GenericRelation(Feedback)

    @property
    def converted(self, content_type="audio/mpeg"):
        """
        Function that converts the uploaded audio to .mp3 and
        returns the converted file

        :param self: A handle to the :class:`models.Document`
        :type self: class: `models.Document`
        :param content_type: content type of the converted file, defaults to "audio/mpeg"
        :type request: content_type

        :return: File containing .mp3 audio
        :rtype: .mp3 File
        """
        super(Document, self).save()
        file_path = self.audio.path
        original_extension = file_path.split(".")[-1]
        mp3_converted_file = AudioSegment.from_file(file_path, original_extension)
        new_path = file_path[:-4] + "-conv.mp3"
        mp3_converted_file.export(new_path, format="mp3", bitrate="44.1k")

        converted_audiofile = File(file=open(new_path, "rb"), name=Path(new_path))
        converted_audiofile.name = Path(new_path).name
        converted_audiofile.content_type = content_type
        converted_audiofile.size = os.path.getsize(new_path)
        os.remove(new_path)
        return converted_audiofile

    def save(self, *args, **kwargs):
        """Overwrite djangos save function to convert audio files
        to mp3 format (orignal file is saved as backup).
        """
        if self.audio:
            self.audio = self.converted
        super(Document, self).save(*args, **kwargs)

    def __str__(self):
        """String representation of Document instance

        :return: title of document instance
        :rtype: str
        """
        return document_to_string(self)

    class Meta:
        """
        Define user readable name of Document
        """

        verbose_name = _("vocabulary")
        verbose_name_plural = _("vocabulary")



from django.contrib.admin.widgets import FilteredSelectMultiple
from django.urls import reverse_lazy


class ManyToManyOverlay(FilteredSelectMultiple):
    """Adds an overlay to the `FilteredSelectMultiple`
    widget to preview images and audios of Document objects.
    Inherits from `django.contrib.admin.widgets.FilteredSelectMultiple`.
    """

    class Media:
        js = (
            "js/jquery.min.js",
            "js/manytomany_overlay.js",
            "js/overlay.js",
            reverse_lazy("javascript-translations"),
        )
        css = {"all": ("css/overlay.css",)}

    def __init__(self, *args, **kwargs):
        """
        Instantiates model by calling `super()` and passing a new `onclick`
        event to the widget.
        """
        return super(ManyToManyOverlay, self).__init__(
            attrs={"onclick": "document_overlay(event);"}, *args, **kwargs
        )



from django.contrib.auth.models import Group
from rest_framework import serializers

from ....cms.models import Discipline


class GroupSerializer(serializers.ModelSerializer):
    """
    Serializer for the Group module. Inherits from
    `serializers.ModelSerializer`.
    """

    total_discipline_children = serializers.SerializerMethodField(
        "get_total_discipline_children"
    )

    class Meta:
        """
        Define model and the corresponding fields
        """

        model = Group
        fields = (
            "id",
            "name",
            "icon",
            "total_discipline_children",
        )

    def get_total_discipline_children(self, obj):
        """Returns the total child count of a group.
        A child itself or one of its sub-children needs to
        contain at least one training set.

        :param disc: Discipline instance
        :type disc: models.Discipline
        :return: sum of children
        :rtype: int
        """
        queryset = Discipline.objects.filter(released=True, created_by=obj.id)
        queryset = [obj for obj in queryset if obj.is_root_node() and obj.is_valid()]
        return len(queryset)



import json

from django.shortcuts import render

from ...cms.models import TrainingSet, Document, DocumentImage, Discipline


def public_upload(request):
    """Public form to upload missing images

    :param request: current user request
    :type request: django.http.request
    :return: rendered response
    :rtype: HttpResponse
    """
    upload_success = False
    if request.method == "POST":
        document = Document.objects.get(id=request.POST.get("inputDocument", None))
        if document:
            uploaded_image = request.FILES.get("inputFile", None)
            if uploaded_image:
                image = DocumentImage(
                    document=document,
                    image=uploaded_image,
                    confirmed=False,
                )
                image.save()
                upload_success = True
    missing_images = Document.objects.values_list(
        "id", "word", "article", "training_sets"
    ).filter(document_image__isnull=True)
    training_sets = (
        TrainingSet.objects.values_list("id", "title")
        .filter(documents__document_image__isnull=True)
        .distinct()
    )
    disciplines = (
        Discipline.objects.values_list("id", "title")
        .order_by("title")
        .filter(training_sets__isnull=False)
        .filter(training_sets__documents__document_image__isnull=True)
        .distinct()
    )
    disc_sets_map = (
        Discipline.objects.values_list("id", "training_sets__id")
        .filter(training_sets__isnull=False)
        .filter(training_sets__documents__document_image__isnull=True)
        .distinct()
    )

    new_map = {}
    for (key, value) in disc_sets_map:
        if key in new_map:
            new_map[key].append(value)
        else:
            new_map[key] = [value]

    context = {
        "documents": json.dumps(list(missing_images)),
        "disciplines": json.dumps(list(disciplines)),
        "disc_sets_map": json.dumps(new_map),
        "training_sets": json.dumps(list(training_sets)),
        "upload_success": upload_success,
    }
    return render(request, "public_upload.html", context)



"""
URL patterns for the first version of the Lunes API
"""
from django.urls import include, path, re_path

from rest_framework import permissions
from rest_framework.versioning import NamespaceVersioning

from drf_spectacular.views import SpectacularAPIView, SpectacularSwaggerView

from ..utils import OptionalSlashRouter
from . import views


#: The namespace for this URL config (see :attr:`django.urls.ResolverMatch.app_name`)
app_name = "v1"

#: Router for dynamic url patterns
router = OptionalSlashRouter()
router.register(r"disciplines", views.DisciplineViewSet, "disciplines")
router.register(
    "disciplines_by_level/",
    views.DisciplineFilteredViewSet,
    "disciplines_overview",
)
router.register(
    r"disciplines_by_level/(?P<discipline_id>[0-9]+)?",
    views.DisciplineFilteredViewSet,
    "disciplines_by_level",
)
router.register(
    r"disciplines_by_group/(?P<group_id>[0-9]+)",
    views.DisciplineFilteredViewSet,
    "discipline_by_group",
)
router.register(
    r"training_set/(?P<discipline_id>[0-9]+)", views.TrainingSetViewSet, "training_set"
)
router.register(
    r"training_sets",
    views.TrainingSetByIdViewSet,
    "training_sets",
)
router.register(
    r"documents/(?P<training_set_id>[0-9]+)", views.DocumentViewSet, "documents"
)

router.register(
    r"document_by_id/(?P<document_id>[0-9]+)", views.DocumentByIdViewSet, "documents"
)
router.register(r"words", views.WordViewSet, "words")
router.register(r"group_info", views.GroupViewSet, "group_by_id")
router.register(r"feedback", views.CreateFeedbackViewSet, "feedback")

#: The url patterns of this module (see :doc:`django:topics/http/urls`)
urlpatterns = [
    path("", include(router.urls)),
    path("schema/", SpectacularAPIView.as_view(), name="schema"),
    path(
        "docs/",
        SpectacularSwaggerView.as_view(
            template_name="swagger_ui.html", url_name="api:v1:schema"
        ),
        name="swagger-ui",
    ),
]



from rest_framework import serializers
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _

from ....cms.models import Feedback


class FeedbackSerializer(serializers.ModelSerializer):
    """
    Serializer for the Feedback module. Inherits from
    'serializers.ModelSerializer'.
    """

    content_type = serializers.SlugRelatedField(
        slug_field="model",
        queryset=ContentType.objects.filter(
            app_label="cms", model__in=["discipline", "trainingset", "document"]
        ),
        error_messages={
            "does_not_exist": _(
                "The content type must be either 'discipline', 'training set' or 'document'."
            ),
        },
    )

    def validate(self, attrs):
        try:
            attrs["content_type"].model_class().objects.get(pk=attrs["object_id"])
        except:
            raise serializers.ValidationError(
                {
                    "object_id": [
                        _("{} with id {} does not exist.").format(
                            attrs["content_type"].name, attrs["object_id"]
                        ),
                    ]
                }
            )
        return attrs

    class Meta:
        """
        Define model and the corresponding fields
        """

        model = Feedback
        fields = (
            "comment",
            "content_type",
            "object_id",
        )



from rest_framework import serializers

from ....cms.models import DocumentImage
from .document_image_list_serializer import DocumentImageListSerializer


class DocumentImageSerializer(serializers.ModelSerializer):
    """
    Serializer for the DocumentImage module. Inherits from
    `serializers.ModelSerializer`.
    """

    class Meta:
        """
        Define model and the corresponding fields
        """

        list_serializer_class = DocumentImageListSerializer
        model = DocumentImage
        fields = ("id", "image")



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class CmsConfig(AppConfig):
    """
    Application settings for the `cms` app, which
    is our main cms app. Inherits from `AppConfig`.
    """

    name = "lunes_cms.cms"
    verbose_name = _("vocabulary management")



"""
A collection of helper methods and classes
"""

import os
import uuid
import string
import pathlib

from html import escape

from django.utils.crypto import get_random_string
from django.utils.html import mark_safe
from django.utils.translation import ugettext as _


def create_ressource_path(parent_dir, filename):
    """Create a file path with a uuid and given parent directory.

    :param parent_dir: parent directory
    :type parent_dir: str
    :param filename: file name
    :type filename: str
    :return: full file path
    :rtype: str
    """
    return os.path.join(parent_dir, str(uuid.uuid1()) + pathlib.Path(filename).suffix)


def get_random_key(length: int = 10, excluded_chars: list = []) -> str:
    """Auxiliary function that creates a random key based on latin letters and digits using
    the passed length. Optionally, it is possible to exclude characters like l and 1.

    :param length: key length, defaults to 10
    :type length: int, optional
    :param excluded_chars: list of characters to be excluded (mixed dtypes possible), defaults to []
    :type excluded_chars: list, optional
    :return: key
    :rtype: str
    """
    choices = string.ascii_letters + string.digits
    for char in excluded_chars:
        choices = choices.replace(str(char), "")
    key = get_random_string(length, choices)
    return key


def document_to_string(doc):
    """Create string representation of a document object

    :param doc: Document object
    :type doc: models.Document
    :return: String representation of document image
    :rtype: str
    """
    alt_words = [str(elem) for elem in doc.alternatives.all()]

    if len(alt_words) > 0:
        alt_words = "(" + ", ".join(alt_words) + ")"
        return "(" + doc.get_article_display() + ") " + doc.word + " " + alt_words
    else:
        return "(" + doc.get_article_display() + ") " + doc.word


def get_child_count(disc):
    """Returns the number of children of a discipline.
    Every child contains at least one training set or is a direct/indirect
    parent of a discipline that contains one.

    :param disc: Discipline instance
    :type disc: models.Discipline
    :return: sum of children
    :rtype: int
    """
    children_counter = 0
    for child in disc.get_children():
        if child.released and get_training_set_count(child) > 0:
            children_counter += 1
    return children_counter


def get_training_set_count(disc):
    """Returns the total number of training sets of a discipline and all its
    child elements.

    :param disc: Discipline instance
    :type disc: models.Discipline
    :return: sum of training sets
    :rtype: int
    """
    training_set_counter = 0
    for child in disc.get_descendants(include_self=True):
        training_set_counter += child.training_sets.count()
    return training_set_counter


def get_image_tag(image):
    """
    Image thumbnail to display a preview of a image

    :param image: The image file
    :type image: ~django.db.models.fields.files.ImageFieldFile

    :return: HTML tag to display an image thumbnail
    :rtype: str
    """
    src = ""
    if (
        image
        and image.storage.exists(image.name)
        and any(image.name.lower().endswith(ext) for ext in [".jpg", ".png"])
    ):
        # The normal src attribute for jpg and png previews
        src = escape(f"/media/{image}")
    # Hide preview if image is empty or has invalid type
    html_cls = "" if src else 'class="hidden"'
    # HTML image tag for previews
    return mark_safe(f'<img src="{src}" width="330" height="auto" {html_cls} />')


def iter_to_string(iter):
    """
    Convert an iterable of objects to a readable string.
    It joins the first elements with commas and separates the last element by "and".

    :param iter: The input iterable
    :type iter: ~collections.abc.Iterable

    :return: The joined list
    :rtype: str
    """
    # Convert iterable to list to support querysets etc.
    lst = list(iter)
    # If the list contains more than 1 element, save the last element for later
    last_element = lst.pop() if len(lst) > 1 else None
    # Join remaining elements with commas and surrounding quotes
    list_str = '"' + '", "'.join(map(str, lst)) + '"'
    # Append the last element with "and"
    if last_element:
        list_str += " " + _("and") + f' "{last_element}"'
    # Return final string
    return list_str



from django.templatetags.static import static
from rest_framework import serializers


class FallbackIconSerializer(serializers.ModelSerializer):
    """
    Serializer for models with an icon field which should provide a fallback value
    """

    icon = serializers.SerializerMethodField()

    def get_icon(self, obj):
        """
        Get the the icon if it exists and a fallback image otherwise

        :param obj: The model instance
        :type obj: ~django.db.models.Model

        :return: The url to the icon
        :rtype: str
        """
        if obj.icon:
            url = obj.icon.url
        else:
            url = static("images/fallback-icon.svg")
        request = self.context.get("request")
        if request:
            return request.build_absolute_uri(url)
        return url



from __future__ import absolute_import, unicode_literals

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from mptt.admin import DraggableMPTTAdmin

from ..models import Static, Discipline


class DisciplineAdmin(DraggableMPTTAdmin):
    """
    Admin Interface to for the Discipline module.
    Inheriting from `mptt.admin.DraggableMPTTAdmin`.
    """

    fields = [
        "released",
        "title",
        "description",
        "icon",
        "image_tag",
        "parent",
        "created_by",
    ]
    readonly_fields = ["created_by", "image_tag"]
    search_fields = ["title"]
    actions = ["delete_selected", "make_released", "make_unreleased"]
    list_per_page = 25

    def save_model(self, request, obj, form, change):
        """
        Overwrite django built-in function to save
        user group and admin satus of model

        :param request: current user request
        :type request: django.http.request
        :param obj: discipline object
        :type obj: models.Discipline
        :param form: employed model form
        :type form: ModelForm
        :param change: True if change on existing model
        :type change: bool
        :raises IndexError: Error when user is not superuser and doesn't belong to any group
        """
        if not change:
            if len(request.user.groups.all()) >= 1:
                obj.created_by = request.user.groups.all()[0]
            elif not request.user.is_superuser:
                raise IndexError("No group assigned. Please add the user to a group")
            obj.creator_is_admin = request.user.is_superuser
        obj.save()

    def get_action_choices(self, request):
        """
        Overwrite django built-in function to modify action choices. The first
        option is dropped since it is a place holder.

        :param request: current user request
        :type request: django.http.request
        :return: modified action choices
        :rtype: dict
        """
        choices = super(DisciplineAdmin, self).get_action_choices(request)
        choices.pop(0)
        return choices

    def get_queryset(self, request):
        """
        Overwrite django built-in function to modify queryset according to user.
        Users that are not superusers only see disciplines of their groups.

        :param request: current user request
        :type request: django.http.request
        :return: adjustet queryset
        :rtype: QuerySet
        """
        qs = super(DisciplineAdmin, self).get_queryset(request)
        if request.user.is_superuser:
            return qs.filter(creator_is_admin=True)
        return qs.filter(created_by__in=request.user.groups.all())

    def get_form(self, request, obj=None, **kwargs):
        """
        Overwrite django built-in function to define custom choices
        in mppt many to many selector for parent disciplines,
        e.g. users should not see disciplines by superusers.
        The function modifies the querysets of the
        corresponding base fields dynamically.

        :param request: current user request
        :type request: django.http.request
        :param obj: django model object, defaults to None
        :type obj: django.db.models, optional
        :return: model form with adjustet querysets
        :rtype: ModelForm
        """
        form = super(DisciplineAdmin, self).get_form(request, obj, **kwargs)
        if not request.user.is_superuser:
            form.base_fields["parent"].queryset = (
                Discipline.objects.filter(
                    created_by__in=request.user.groups.all(),
                    training_sets__isnull=True,
                )
                .order_by("title")
                .order_by("level")
            )
        else:
            form.base_fields["parent"].queryset = (
                Discipline.objects.filter(
                    creator_is_admin=True,
                    training_sets__isnull=True,
                )
                .order_by("title")
                .order_by("level")
            )
        return form

    @admin.action(description=_("Release selected disciplines"))
    def make_released(self, request, queryset):
        """
        Action to release discipline objects. It sets the
        corresponding boolean field to true.

        :param request: current user request
        :type request: django.http.request
        :param queryset: current queryset
        :type queryset: QuerySet
        """
        queryset.update(released=True)

    @admin.action(description=_("Unrelease selected disciplines"))
    def make_unreleased(self, request, queryset):
        """
        Action to hide discipline objects. It sets the
        corresponding boolean field to false.

        :param request: current user request
        :type request: django.http.request
        :param queryset: current queryset
        :type queryset: QuerySet
        """
        queryset.update(released=False)

    def creator_group(self, obj):
        """
        Include creator group of discipline in list display

        :param obj: Discipline object
        :type obj: models.Discipline
        :return: Either static admin group or user group
        :rtype: str
        """
        if obj.creator_is_admin:
            return Static.admin_group
        elif obj.created_by:
            return obj.created_by
        else:
            return None

    creator_group.short_description = _("creator group")

    class Media:
        js = ("js/image_preview.js",)



from html import escape

from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.utils.safestring import mark_safe
from django.utils.text import Truncator


class Feedback(models.Model):
    """
    Model to store feedback about disciplines, training sets and vocabulary words
    """

    content_type = models.ForeignKey(
        ContentType,
        limit_choices_to=models.Q(
            app_label="cms", model__in=["discipline", "trainingset", "document"]
        ),
        on_delete=models.CASCADE,
        verbose_name=_("content type"),
        help_text=_("The content type this feedback entry refers to."),
    )
    object_id = models.PositiveIntegerField(
        verbose_name=_("object id"),
        help_text=_("The id of the object this feedback entry refers to."),
    )
    content_object = GenericForeignKey("content_type", "object_id")
    content_object.short_description = _("refers to")

    comment = models.TextField(
        verbose_name=_("comment"),
    )

    read_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="feedback",
        verbose_name=_("marked as read by"),
        help_text=_(
            "The user who marked this feedback as read. If the feedback is unread, this field is empty."
        ),
    )

    created_date = models.DateTimeField(
        auto_now_add=True,
        verbose_name=_("submitted on"),
        help_text=_("The time and date when the feedback was submitted."),
    )

    def content_object_link(self):
        """
        Include a link to the edit form of the content object in list display

        :return: Link to the edit form of the content object
        :rtype: str
        """
        # Get link to the admin form if the content object
        admin_edit_link = reverse(
            f"admin:cms_{self.content_type.model}_change", args=[self.object_id]
        )
        # Escape label because string is marked as safe afterwards
        escaped_label = escape(str(self.content_object))
        # Mark string as safe to allow html link tag
        return mark_safe(f"<a href={admin_edit_link}>{escaped_label}</a>")

    content_object_link.short_description = _("refers to")

    def __str__(self):
        """
        String representation of feedback instance

        :return: Truncated feedback comment
        :rtype: str
        """
        return Truncator(self.comment).chars(80)

    class Meta:
        #: The verbose name of the model
        verbose_name = _("feedback")
        #: The plural verbose name of the model
        verbose_name_plural = _("feedback entries")



from django.db import migrations, models
import lunes_cms.cms.models.static


class Migration(migrations.Migration):

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        ("cms", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="group",
            name="icon",
            field=models.ImageField(
                blank=True,
                upload_to=lunes_cms.cms.models.static.convert_umlaute_images,
                verbose_name="icon",
            ),
        ),
    ]

    def mutate_state(self, project_state, preserve=True):
        """
        This is a workaround that allows to store ``auth``
        migration outside the directory it should be stored.
        Takes a ProjectState and returns a new one with the migration's operations applied to it.
        Preserves the original object state by default and will return a mutated state from a copy.
        """
        app_label = self.app_label
        self.app_label = "auth"
        state = super(Migration, self).mutate_state(project_state, preserve)
        self.app_label = app_label
        return state

    def apply(self, project_state, schema_editor, collect_sql=False):
        """
        Same workaround as described in ``mutate_state`` method.
        Takes a project_state representing all migrations prior to this one
        and a schema_editor for a live database and applies the migration in a forwards order.
        Returns the resulting project state for efficient re-use by following Migrations.
        """
        app_label = self.app_label
        self.app_label = "auth"
        state = super(Migration, self).apply(project_state, schema_editor, collect_sql)
        self.app_label = app_label
        return state



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class HelpConfig(AppConfig):
    """
    Application settings for the `help` app,
    which is the app providing the public upload.
    Inherits from `AppConfig`.
    """

    name = "lunes_cms.help"
    verbose_name = _("help")



from django.contrib.auth.models import Group
from django.db import models
from django.utils.translation import ugettext_lazy as _

from .static import convert_umlaute_images

Group.add_to_class(
    "icon",
    models.ImageField(
        upload_to=convert_umlaute_images, blank=True, verbose_name=_("icon")
    ),
)



"""
This module contains the model serializers, see :doc:`django:topics/serialization`.
"""

from .alternative_word_serializer import AlternativeWordSerializer
from .discipline_serializer import DisciplineSerializer
from .document_image_list_serializer import DocumentImageListSerializer
from .document_image_serializer import DocumentImageSerializer
from .document_serializer import DocumentSerializer
from .feedback_serializer import FeedbackSerializer
from .group_serializer import GroupSerializer
from .training_set_serializer import TrainingSetSerializer



# Generated by Django 3.2.13 on 2022-07-24 19:33

import django.core.validators
from django.db import migrations, models
import lunes_cms.cms.models.group_api_key


class Migration(migrations.Migration):

    dependencies = [
        ("cms", "0005_auto_create_id"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="groupapikey",
            name="hashed_key",
        ),
        migrations.RemoveField(
            model_name="groupapikey",
            name="prefix",
        ),
        migrations.RemoveField(
            model_name="groupapikey",
            name="id",
        ),
        migrations.AddField(
            model_name="groupapikey",
            name="id",
            field=models.AutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.RenameField(
            model_name="groupapikey",
            old_name="organization",
            new_name="group",
        ),
        migrations.AlterField(
            model_name="groupapikey",
            name="group",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="api_keys",
                to="auth.group",
                verbose_name="group",
            ),
        ),
        migrations.RenameField(
            model_name="groupapikey",
            old_name="name",
            new_name="token",
        ),
        migrations.AlterField(
            model_name="groupapikey",
            name="token",
            field=models.CharField(
                default=lunes_cms.cms.models.group_api_key.generate_default_token,
                help_text="10-50 characters, only digits and upper case letters allowed.",
                max_length=50,
                unique=True,
                validators=[
                    django.core.validators.MinLengthValidator(10),
                    django.core.validators.RegexValidator(
                        "^[0-9A-Z]*$", "Only digits and upper case letters allowed"
                    ),
                ],
                verbose_name="token",
            ),
        ),
        migrations.RenameField(
            model_name="groupapikey",
            old_name="created",
            new_name="creation_date",
        ),
        migrations.AlterField(
            model_name="groupapikey",
            name="creation_date",
            field=models.DateTimeField(auto_now_add=True, verbose_name="creation date"),
        ),
        migrations.AlterField(
            model_name="groupapikey",
            name="expiry_date",
            field=models.DateTimeField(
                blank=True,
                help_text="Once API key expires, clients cannot use it anymore.",
                null=True,
                verbose_name="expiration date",
            ),
        ),
        migrations.AlterField(
            model_name="groupapikey",
            name="revoked",
            field=models.BooleanField(
                blank=True,
                default=False,
                help_text="If the API key is revoked, clients cannot use it anymore.",
                verbose_name="revoked",
            ),
        ),
    ]



from .discipline_admin import DisciplineAdmin
from .training_set_admin import TrainingSetAdmin
from .document_admin import DocumentAdmin
from .group_api_key_admin import GroupAPIKeyAdmin
from .feedback_admin import FeedbackAdmin

__all__ = [
    "DisciplineAdmin",
    "TrainingSetAdmin",
    "DocumentAdmin",
    "GroupAPIKeyAdmin",
    "FeedbackAdmin",
]



from __future__ import absolute_import, unicode_literals

from django.contrib import admin
from django.db.models import Case, Exists, IntegerField, OuterRef, Value, When
from django.utils.translation import ugettext_lazy as _

from ..list_filter import (
    DocumentDisciplineListFilter,
    DocumentTrainingSetListFilter,
    ApprovedImageListFilter,
    AssignedListFilter,
)
from ..models import Static, DocumentImage
from .document_image_admin import DocumentImageAdmin
from .alternative_word_admin import AlternativeWordAdmin


SUPERUSER_ONLY_LIST_FILTERS = [ApprovedImageListFilter]


class DocumentAdmin(admin.ModelAdmin):
    """
    Admin Interface to for the Document module.
    Inheriting from `admin.ModelAdmin`.
    """

    exclude = ("article_plural", "creator_is_admin")
    readonly_fields = ("created_by",)
    search_fields = ["word", "alternatives__alt_word"]
    inlines = [DocumentImageAdmin, AlternativeWordAdmin]
    ordering = ["word", "creation_date"]
    list_display = (
        "word",
        "word_type",
        "article_display",
        "related_training_set",
        "has_audio",
        "has_image",
        "creator_group",
        "creation_date",
    )
    list_filter = (
        DocumentDisciplineListFilter,
        DocumentTrainingSetListFilter,
        ApprovedImageListFilter,
        AssignedListFilter,
    )
    list_per_page = 25

    def save_model(self, request, obj, form, change):
        """
        Overwrite django built-in function to save
        user group and admin satus of model

        :param request: current user request
        :type request: django.http.request
        :param obj: document object
        :type obj: models.Document
        :param form: employed model form
        :type form: ModelForm
        :param change: True if change on existing model
        :type change: bool
        :raises IndexError: Error when user is not superuser and doesn't belong to any group
        """
        if not change:
            if len(request.user.groups.all()) >= 1:
                obj.created_by = request.user.groups.all()[0]
            elif not request.user.is_superuser:
                raise IndexError("No group assigned. Please add the user to a group")
            obj.creator_is_admin = request.user.is_superuser
        obj.save()

    def get_action_choices(self, request):
        """
        Overwrite django built-in function to modify action choices. The first
        option is dropped since it is a place holder.

        :param request: current user request
        :type request: django.http.request
        :return: modified action choices
        :rtype: dict
        """
        choices = super(DocumentAdmin, self).get_action_choices(request)
        choices.pop(0)
        return choices

    def get_queryset(self, request):
        """
        Overwrite django built-in function to modify queryset according to user.
        Users that are not superusers only see documents of their groups.

        :param request: current user request
        :type request: django.http.request
        :return: adjustet queryset
        :rtype: QuerySet
        """
        qs = (
            super(DocumentAdmin, self)
            .get_queryset(request)
            .annotate(
                has_image=Exists(DocumentImage.objects.filter(document=OuterRef("pk"))),
                has_confirmed_image=Exists(
                    DocumentImage.objects.filter(
                        document=OuterRef("pk"), confirmed=True
                    )
                ),
                image_sort=Case(
                    When(has_confirmed_image=True, then=Value(2)),
                    When(has_image=True, then=Value(1)),
                    default=Value(0),
                    output_field=IntegerField(),
                ),
            )
        )
        if request.user.is_superuser:
            return qs.filter(creator_is_admin=True)
        return qs.filter(created_by__in=request.user.groups.all())

    def related_training_set(self, obj):
        """
        Display related training sets in list display

        :param obj: Document object
        :type obj: models.Document
        :return: comma seperated list of related training sets
        :rtype: str
        """
        return ", ".join([child.title for child in obj.training_sets.all()])

    related_training_set.short_description = _("training set")

    def creator_group(self, obj):
        """
        Include creator group of discipline in list display

        :param obj: Document object
        :type obj: models.Document
        :return: Either static admin group or user group
        :rtype: str
        """
        if obj.creator_is_admin:
            return Static.admin_group
        elif obj.created_by:
            return obj.created_by
        else:
            return None

    creator_group.short_description = _("creator group")
    creator_group.admin_order_field = "created_by"

    def has_audio(self, obj):
        """
        Include in list display whether a document has an audio file.

        :param obj: Document object
        :type obj: models.Document
        :return: Either static admin group or user group
        :rtype: str
        """
        if obj.audio:
            return True
        else:
            return False

    has_audio.boolean = True
    has_audio.short_description = _("audio")
    has_audio.admin_order_field = "audio"

    def has_image(self, obj):
        """
        Include in list display whether a document has an image file.

        :param obj: Document object
        :type obj: models.Document

        :return: Whether the document has at least one confirmed image
        :rtype: bool
        """
        if obj.has_confirmed_image:
            return True
        if obj.has_image:
            return None
        return False

    has_image.boolean = True
    has_image.short_description = _("image")
    has_image.admin_order_field = "image_sort"

    def article_display(self, obj):
        """
        Include article of document in list display

        :param obj: Document object
        :type obj: models.Document
        :return: Either static admin group or user group
        :rtype: str
        """
        return obj.get_article_display()

    article_display.short_description = _("article")

    def get_list_filter(self, request):
        """Override djangos get_list_filter function
        to remove specific list filters that are only relevant
        for super users.

        :param request: current request
        :type request: django.http.request
        :param obj: [description], defaults to None
        :type obj: django.db.models, optional
        :return: custom fields list
        :rtype: list[str]
        """
        if request.user.is_superuser:
            return self.list_filter
        # remove filters that are only relevant for super users
        filters = [f for f in self.list_filter if f not in SUPERUSER_ONLY_LIST_FILTERS]
        return tuple(filters)

    class Media:
        js = ("js/image_preview.js",)



from django.contrib import admin

from ..models import AlternativeWord


class AlternativeWordAdmin(admin.StackedInline):
    """
    Admin Interface to for the AlternativeWord module.
    Inheriting from `admin.StackedInline`.
    """

    model = AlternativeWord
    search_fields = ["alt_word"]
    autocomplete_fields = ["document"]
    insert_after = "autocomplete_fields"
    extra = 0



"""
WSGI config for lunes-cms project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/2.2/howto/deployment/wsgi/
"""

import configparser
import os

from django.core.wsgi import get_wsgi_application


def application(environ, start_response):
    """
    This returns the WSGI callable

    :param environ: The environment variables
    :type environ: dict

    :param start_response: A function which starts the response
    :type start_response: ~collections.abc.Callable

    :return: The WSGI callable
    :rtype: ~django.core.handlers.WSGIHandler
    """
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "lunes_cms.core.settings")

    # Read config from config file
    config = configparser.ConfigParser(interpolation=None)
    config.read("/etc/lunes-cms.ini")
    for section in config.sections():
        for KEY, VALUE in config.items(section):
            os.environ.setdefault(f"LUNES_CMS_{KEY.upper()}", VALUE)

    # Read config from environment
    for key in environ:
        if key.startswith("LUNES_CMS_"):
            os.environ[key] = environ[key]

    _application = get_wsgi_application()

    return _application(environ, start_response)



from html import escape
from string import digits, ascii_uppercase

from django.contrib.auth.models import Group
from django.core.exceptions import PermissionDenied
from django.core.validators import MinLengthValidator, RegexValidator
from django.db import models
from django.db.models import Q
from django.utils.crypto import get_random_string
from django.utils.html import mark_safe
from django.utils.translation import ugettext_lazy as _
from django.utils.timezone import now

from qr_code.qrcode.maker import make_qr_code_url_with_args


def generate_default_token():
    """
    Generate a default random token of upper case letters and digits

    :return: A default token
    :rtype: str
    """
    return get_random_string(
        length=10, allowed_chars=digits + ascii_uppercase.replace("O", "")
    )


class GroupAPIKey(models.Model):
    """
    Model that handles api keys associated with a user group.
    """

    group = models.ForeignKey(
        Group,
        on_delete=models.CASCADE,
        verbose_name=_("group"),
        related_name="api_keys",
    )
    token = models.CharField(
        unique=True,
        max_length=50,
        verbose_name=_("token"),
        help_text=_("10-50 characters, only digits and upper case letters allowed."),
        validators=[
            MinLengthValidator(10),
            RegexValidator(
                r"^[0-9A-Z]*$", _("Only digits and upper case letters allowed")
            ),
        ],
        default=generate_default_token,
    )
    revoked = models.BooleanField(
        blank=True,
        default=False,
        verbose_name=_("revoked"),
        help_text=_("If the API key is revoked, clients cannot use it anymore."),
    )
    expiry_date = models.DateTimeField(
        blank=True,
        null=True,
        verbose_name=_("expiration date"),
        help_text=_("Once API key expires, clients cannot use it anymore."),
    )
    creation_date = models.DateTimeField(
        auto_now_add=True, verbose_name=_("creation date")
    )

    def is_valid(self):
        """
        An API key is valid if it is not revoked and the exiry date (if set) is not in the past.

        :return: Whether the API key is valid
        :rtype: bool
        """
        return not self.revoked and (not self.expiry_date or self.expiry_date > now())

    is_valid.boolean = True
    is_valid.short_description = _("valid")

    @property
    def qr_code_url(self):
        """
        :return: URL to the QR code for the token
        :rtype: str
        """
        return make_qr_code_url_with_args(
            data=self.token, qr_code_args={"image_format": "png"}
        )

    def qr_code_link(self, link, title=None):
        """
        :param link: The link text/image (needs to be properly escaped)
        :type link: str

        :param title: The link title (optional, needs to be properly escaped)
        :type title: str

        :return: HTML Link to the QR code for the token
        :rtype: str
        """
        return mark_safe(
            f'<a download="Lunes-QR-code-{escape(self.group.name)}.jpg" href="{escape(self.qr_code_url)}" title="{title or link}">{link}</a>'
        )

    def qr_code_download_link(self):
        """
        :return: Download link to the QR code for the token
        :rtype: str
        """
        return self.qr_code_link(escape(_("Download"))) if self.is_valid() else None

    qr_code_download_link.short_description = _("QR code")

    def qr_code(self):
        """
        :return: QR code image for the token with download link
        :rtype: str
        """
        return (
            self.qr_code_link(
                mark_safe(
                    f'<img src="{escape(self.qr_code_url)}" width="330" height="auto" /></a>'
                ),
                escape(_("Download QR code")),
            )
            if self.is_valid()
            else "-"
        )

    qr_code.short_description = _("QR code")

    @classmethod
    def get_from_token(cls, token):
        try:
            return cls.objects.get(
                Q(token=token, revoked=False)
                & (Q(expiry_date__isnull=True) | Q(expiry_date__gt=now()))
            )
        except cls.DoesNotExist:
            raise PermissionDenied()

    def __str__(self):
        """
        :return: The canonical string representation of an API key
        :rtype: str
        """
        return _('{}: Token for the group "{}"').format(self.token, self.group)

    class Meta:
        verbose_name = _("API Key")
        verbose_name_plural = _("API Keys")



from rest_framework import mixins, viewsets

from ..serializers import FeedbackSerializer


class CreateFeedbackViewSet(mixins.CreateModelMixin, viewsets.GenericViewSet):
    """
    A viewset that provides only the ``create`` action for feedback elements.
    """

    serializer_class = FeedbackSerializer



from rest_framework import viewsets
from django.core.exceptions import PermissionDenied
from django.db.models import Count, Q

from ....cms.models import TrainingSet, GroupAPIKey
from ...utils import get_key
from ..serializers import TrainingSetSerializer


class TrainingSetByIdViewSet(viewsets.ReadOnlyModelViewSet):
    """
    Defines a view set for a TrainingSet module of a given id.
    Inherits from `viewsets.ReadOnlyModelViewSet` and defines queryset
    and serializers.
    """

    serializer_class = TrainingSetSerializer

    def get_queryset(self):
        """
        Defining custom queryset

        :param self: A handle to the :class:`TrainingSetViewSet`
        :type self: class

        :return: (filtered) queryset
        :rtype: QuerySet
        """
        if getattr(self, "swagger_fake_view", False):
            return TrainingSet.objects.none()
        queryset = TrainingSet.objects.filter(released=True).annotate(
            total_documents=Count(
                "documents",
                filter=Q(documents__document_image__confirmed=True),
                distinct=True,
            )
        )
        key = get_key(self.request)
        if key:
            queryset = queryset.filter(created_by=GroupAPIKey.get_from_token(key).group)
        else:
            queryset = queryset.filter(creator_is_admin=True)
        return queryset



from django.db.models import Q

from rest_framework import viewsets
from rest_framework.authentication import SessionAuthentication, BasicAuthentication

from ....cms.models import Document, GroupAPIKey
from ...utils import get_key
from ..serializers import DocumentSerializer


class WordViewSet(viewsets.ModelViewSet):
    """
    Retrieve the list of all default words/documents or get a single record by appending the id of the requested word.
    """

    serializer_class = DocumentSerializer
    http_method_names = ["get"]
    authentication_classes = [SessionAuthentication, BasicAuthentication]

    def get_queryset(self):
        """
        Get the queryset of words/documents

        :return: The queryset of words
        :rtype: ~django.db.models.query.QuerySet
        """
        if getattr(self, "swagger_fake_view", False):
            return Document.objects.none()
        # Get base queryset
        queryset = Document.objects.filter(
            document_image__confirmed=True,
            training_sets__released=True,
            training_sets__discipline__released=True,
        )
        # Get API key of current request
        key = get_key(self.request)
        if key:
            # If the key is given, get the corresponding group
            api_key_object = GroupAPIKey.get_from_token(key)
            # Return all public words as well as those of the group with the given key
            queryset = queryset.filter(
                Q(creator_is_admin=True) | Q(created_by=api_key_object.group)
            )
        else:
            # If no key is given, return all public words
            queryset = queryset.filter(creator_is_admin=True)
        return (
            queryset.select_related().distinct()
            # Sort queryset to make output order deterministic for testing
            .order_by("id")
        )



"""
Context processors pass additional variables to templates (see :ref:`context-processors`).
"""
from ..cms.models import Feedback


# pylint: disable=unused-variable
def feedback_processor(request):
    """
    This context processor injects the number of unread feedback entries into the template context.

    :param request: The current http request
    :type request: ~django.http.HttpRequest

    :return: The template context containing the number of unread feedback entries
    :rtype: dict
    """
    return {"unread_feedback_cnt": Feedback.objects.filter(read_by=None).count()}



# Generated by Django 3.2.13 on 2022-07-25 14:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cms", "0004_feedback"),
    ]

    operations = [
        migrations.AlterField(
            model_name="alternativeword",
            name="id",
            field=models.AutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="discipline",
            name="id",
            field=models.AutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="document",
            name="id",
            field=models.AutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="documentimage",
            name="id",
            field=models.AutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="trainingset",
            name="id",
            field=models.AutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]



from django.contrib import admin


class GroupAPIKeyAdmin(admin.ModelAdmin):
    list_display = [
        "token",
        "group",
        "is_valid",
        "expiry_date",
        "qr_code_download_link",
    ]
    search_fields = ["group"]
    readonly_fields = ["is_valid", "qr_code"]
    list_per_page = 25



from .discipline_filtered_viewset import DisciplineFilteredViewSet
from .discipline_viewset import DisciplineViewSet
from .document_by_id_viewset import DocumentByIdViewSet
from .document_viewset import DocumentViewSet
from .group_viewset import GroupViewSet
from .training_set_viewset import TrainingSetViewSet
from .training_set_by_id_viewset import TrainingSetByIdViewSet
from .word_viewset import WordViewSet
from .feedback_viewset import CreateFeedbackViewSet



from django.contrib import admin

from ..models import DocumentImage


class DocumentImageAdmin(admin.StackedInline):
    """
    Admin Interface to for the DocumentImage module.
    Inheriting from `admin.StackedInline`.
    """

    model = DocumentImage
    fields = ["image", "image_tag"]
    superuser_fields = ["confirmed"]
    readonly_fields = ["image_tag"]
    autocomplete_fields = ["document"]
    insert_after = "autocomplete_fields"
    extra = 0

    def get_fields(self, request, obj=None):
        """Override djangos get_fields function
        to add custom superuser fields to the
        admin interface if the user has the corresponding
        access rights.

        :param request: current request
        :type request: django.http.request
        :param obj: [description], defaults to None
        :type obj: django.db.models, optional
        :return: custom fields list
        :rtype: list[str]
        """
        if request.user.is_superuser and self.superuser_fields:
            return (self.fields or tuple()) + self.superuser_fields
        return super(DocumentImageAdmin, self).get_fields(request, obj)



# Generated by Django 3.2.13 on 2022-07-05 11:32

from django.db import migrations, models
import django.db.models.deletion


# pylint: disable=unused-argument
def link_created_by(apps, schema_editor):
    """
    Convert the ``created_by`` char field to a foreign key

    :param apps: The configuration of installed applications
    :type apps: ~django.apps.registry.Apps

    :param schema_editor: The database abstraction layer that creates actual SQL code
    :type schema_editor: ~django.db.backends.base.schema.BaseDatabaseSchemaEditor
    """
    Document = apps.get_model("cms", "Document")
    Group = apps.get_model("auth", "Group")
    for document in Document.objects.filter(created_by__isnull=False):
        group, created = Group.objects.get_or_create(name=document.created_by)
        document.created_by_link = group
        document.save()


# pylint: disable=unused-argument
def reverse_created_by(apps, schema_editor):
    """
    Convert the ``created_by`` field from foreign key to a char field

    :param apps: The configuration of installed applications
    :type apps: ~django.apps.registry.Apps

    :param schema_editor: The database abstraction layer that creates actual SQL code
    :type schema_editor: ~django.db.backends.base.schema.BaseDatabaseSchemaEditor
    """
    Document = apps.get_model("cms", "Document")
    Group = apps.get_model("auth", "Group")
    for document in Document.objects.filter(created_by_link__isnull=False):
        group, created = Group.objects.get_or_create(id=document.created_by_link.id)
        document.created_by = group.name
        document.save()


class Migration(migrations.Migration):

    dependencies = [
        ("cms", "0006_convert_group_api_key"),
    ]

    operations = [
        migrations.AddField(
            model_name="document",
            name="created_by_link",
            field=models.ForeignKey(
                blank=True,
                max_length=255,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="auth.group",
                verbose_name="created by",
            ),
        ),
        migrations.RunPython(link_created_by, reverse_created_by),
        migrations.RemoveField(
            model_name="document",
            name="created_by",
        ),
        migrations.RenameField(
            model_name="document",
            old_name="created_by_link",
            new_name="created_by",
        ),
    ]



"""
A collection of helper methods and classes
"""

from django.core.exceptions import PermissionDenied
from django.db.models import Count, Q

from rest_framework import routers

from ..cms.models import Discipline, GroupAPIKey
from ..cms.utils import get_child_count


class OptionalSlashRouter(routers.DefaultRouter):
    """
    Custom router to allow routes with and without trailing slash to work without redirects
    """

    def __init__(self):
        super().__init__()
        self.trailing_slash = "/?"


def get_key(request, keyword="Api-Key"):
    """Retrieve API Key from Authorization header of http request.
    Optionally, a custom keyword can be specified. The function
    espects the key to be delivered as follows:
    {"Authorization": "<keyword> <api-key>}

    :param request: get request
    :type request: HttpRequest
    :param keyword: keyword for api key in authorization header, defaults to "Api-Key"
    :type keyword: str, optional
    :return: api key in authorization header
    :rtype: str
    """
    authorization = request.META.get("HTTP_AUTHORIZATION")
    if not authorization:
        return None
    try:
        _, key = authorization.split("{} ".format(keyword))
    except ValueError:
        key = None
    return key


def get_filtered_discipline_queryset(discipline_view_set):
    """Returns child disciplines belonging to the discipline id
    of the passed discipline view set. Only released and non-empty
    objects are returned. The number of training sets contained by
    a child is annotated as well.

    :param discipline_view_set: A handle to the :class:`DisciplineViewSet`
    :type discipline_view_set: class
    :return: (filtered) queryset
    :rtype: QuerySet
    """
    queryset = Discipline.objects.filter(
        Q(released=True)
        & Q(
            id__in=Discipline.objects.get(
                id=discipline_view_set.kwargs["discipline_id"]
            ).get_children()
        )
    ).annotate(
        total_training_sets=Count(
            "training_sets", filter=Q(training_sets__released=True)
        ),
    )
    queryset = get_non_empty_disciplines(queryset)
    return queryset


def get_overview_discipline_queryset():
    """Returns the general disciplines created by super users if the are
    root nodes and recursively either has at least one sub-discipline or one
    training set. Additionally, they need to be released by the creator group.

    :return: (filtered) queryset
    :rtype: QuerySet
    """
    queryset = Discipline.objects.filter(
        Q(released=True) & Q(creator_is_admin=True)
    ).annotate(
        total_training_sets=Count(
            "training_sets", filter=Q(training_sets__released=True)
        ),
    )
    queryset = [obj for obj in queryset if obj.is_root_node()]
    queryset = get_non_empty_disciplines(queryset)
    return queryset


def get_discipline_by_group_queryset(discipline_view_set):
    """Returns overview of disciplines for a given group id, which must be
    in the keyword arguments of the passed discipline view set. All elements are
    root nodes and recursively either have at least one sub-discipline or one
    training set. Additionally, they need to be released by the creator group.

    :param discipline_view_set: A handle to the :class:`DisciplineViewSet`
    :type discipline_view_set: class
    :return: (filtered) queryset
    :rtype: QuerySet
    """
    queryset = Discipline.objects.filter(
        released=True, created_by=discipline_view_set.kwargs["group_id"]
    ).annotate(
        total_training_sets=Count(
            "training_sets", filter=Q(training_sets__released=True)
        ),
    )
    queryset = [obj for obj in queryset if obj.is_root_node() and obj.is_valid()]
    return queryset


def get_non_empty_disciplines(queryset):
    """
    Filters a discipline queryset so that every element recursively either have
    at least one sub-discipline or one training set.

    :param queryset: Queryset of `~lunes_cms.cms.models.Discipline` objects
    :type queryset: QuerySet
    :return: (filtered) queryset
    :rtype: QuerySet
    """
    queryset = [
        obj
        for obj in queryset
        if get_child_count(obj) > 0
        or obj.training_sets.filter(released=True).count() > 0
    ]
    return queryset


def check_group_object_permissions(request, group_id):
    """Function to check if the API-Key of the passed request object
    matches one of the hashed keys stored in the database of the
    corresponding group id.

    :param request: current request
    :type request: HttpRequest

    :param group_id: group id
    :type group_id: str

    :raises PermissionDenied: Exception if no API-Key is delivered
    :raises PermissionDenied: Exception if API-Key doesn't belong to passed group id
    """
    key = get_key(request)
    api_key_object = GroupAPIKey.get_from_token(key)
    if api_key_object.group_id != int(group_id):
        raise PermissionDenied()



import logging


class ColorFormatter(logging.Formatter):
    """
    Logging Formatter to add colors
    """

    #: The bash color codes for the different logging levels
    COLORS = {
        logging.DEBUG: 36,  # cyan
        logging.INFO: 34,  # blue
        logging.WARNING: 33,  # yellow
        logging.ERROR: 31,  # red
        logging.CRITICAL: 31,  # red
    }

    def format(self, record):
        """
        Format the specified record as colored text (see :meth:`python:logging.Formatter.format`).

        :param record: The log record
        :type record: ~logging.LogRecord

        :return: The formatted logging message
        :rtype: str
        """
        # Define color escape sequence
        color = f"\x1b[0;{self.COLORS.get(record.levelno)}m"
        # Make level name bold
        fmt = self._fmt.replace("{levelname}", "\x1b[1m{levelname}" + color)
        # Make entire line colored
        # pylint: disable=protected-access
        self._style._fmt = color + fmt + "\x1b[0m"
        return super().format(record)



from rest_framework import mixins, viewsets

from ....cms.models import Discipline
from ...utils import (
    get_filtered_discipline_queryset,
    get_discipline_by_group_queryset,
    get_overview_discipline_queryset,
    check_group_object_permissions,
)
from ..serializers import DisciplineSerializer


class DisciplineFilteredViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    """
    Defines a view set for the Discipline module, optionally filtered respected to
    the different mptt levels or a group id.
    If no discipline id is given, all root elements will be displayed.
    Inherits from `viewsets.ModelViewSet` and defines queryset
    and serializers.
    """

    queryset = Discipline.objects.all()
    serializer_class = DisciplineSerializer

    def get_queryset(self):
        """
        Defining custom queryset

        :param self: A handle to the :class:`DisciplineViewSet`
        :type self: class

        :return: (filtered) queryset
        :rtype: QuerySet
        """
        if getattr(self, "swagger_fake_view", False):
            return Discipline.objects.none()
        if "discipline_id" in self.kwargs:
            discipline_infos = Discipline.objects.filter(
                id=self.kwargs["discipline_id"]
            ).values_list("created_by_id", "creator_is_admin")
            group_id = discipline_infos[0][0]
            is_admin = discipline_infos[0][1]
            if group_id and not is_admin:
                check_group_object_permissions(self.request, group_id)
            queryset = get_filtered_discipline_queryset(self)
        elif "group_id" in self.kwargs:
            check_group_object_permissions(self.request, self.kwargs["group_id"])
            queryset = get_discipline_by_group_queryset(self)
        else:
            queryset = get_overview_discipline_queryset()
        return queryset



"""
URL patterns for the Lunes API
"""
from django.urls import include, path


#: The namespace for this URL config (see :attr:`django.urls.ResolverMatch.app_name`)
app_name = "api"

#: The url patterns of this module (see :doc:`django:topics/http/urls`)
urlpatterns = [
    path("", include("lunes_cms.api.v1.urls", namespace="default")),
    path("v1/", include("lunes_cms.api.v1.urls", namespace="v1")),
]



from django.core.exceptions import PermissionDenied
from django.db.models import Q, Count

from rest_framework import viewsets

from ....cms.models import Discipline, GroupAPIKey
from ...utils import get_key
from ..serializers import DisciplineSerializer


class DisciplineViewSet(viewsets.ModelViewSet):
    """
    Retrieve disciplines, either all global disciplines or filtered by the given API key.
    Get a single record by appending the id of the requested discipline.
    """

    serializer_class = DisciplineSerializer
    http_method_names = ["get"]

    def get_queryset(self):
        """
        Get the queryset of disciplines - either those created by admins or by the group of the given key

        :return: The queryset of disciplines
        :rtype: ~django.db.models.query.QuerySet
        """
        if getattr(self, "swagger_fake_view", False):
            return Discipline.objects.none()
        queryset = Discipline.objects.filter(released=True)
        key = get_key(self.request)
        if key:
            # If the key is given, get the corresponding group
            api_key_object = GroupAPIKey.get_from_token(key)
            # Return all disciplines of the group with the given key
            self.queryset = queryset.filter(created_by=api_key_object.group)
        else:
            # If no key is given, return all admin disciplines
            self.queryset = queryset.filter(creator_is_admin=True)
        # Return annotated queryset
        return self.queryset.annotate(
            total_training_sets=Count(
                "training_sets", filter=Q(training_sets__released=True)
            ),
            total_discipline_children=Count("children"),
        )



"""
Map paths to view functions.
Defines custom schema views and a router that
handles the url patterns described in the `README.md` file
"""
from django.urls import path

from . import views


#: The url patterns of this module (see :doc:`django:topics/http/urls`)
urlpatterns = [
    path("public_upload", views.public_upload, name="public_upload"),
]



from rest_framework import serializers

from ....cms.models import Discipline
from ....cms.utils import get_child_count
from .fallback_icon_serializer import FallbackIconSerializer


class DisciplineSerializer(FallbackIconSerializer):
    """
    Serializer for the Discipline module. Inherits from
    `FallbackIconSerializer`.
    """

    total_training_sets = serializers.IntegerField()
    total_discipline_children = serializers.SerializerMethodField(
        "get_total_discipline_children"
    )
    nested_training_sets = serializers.ListSerializer(
        child=serializers.IntegerField(), source="get_nested_training_sets"
    )

    class Meta:
        """
        Define model and the corresponding fields
        """

        model = Discipline
        fields = (
            "id",
            "title",
            "description",
            "icon",
            "created_by",
            "total_training_sets",
            "total_discipline_children",
            "nested_training_sets",
        )

    def get_total_discipline_children(self, obj):
        """Returns the total child count by calling
        utils.get_child_count(obj).


        :param disc: Discipline instance
        :type disc: models.Discipline
        :return: sum of children
        :rtype: int
        """
        return get_child_count(obj)
